/*
*  C Implementation: sh.h
*
* Description: All variables needed to interface with screen handler
*
*
* Author: Ricardo Cantu <ricky@smartcsc.com>, (C) 2006
*
* Copyright: See COPYING file that comes with this distribution
*
*/

extern int      tish ();      /* Main sh call */

#define ENG_SH_RD      0x4452   /* Read */
#define ENG_SH_RC      0x4352   /* Read Control */
#define ENG_SH_RR      0x5252   /* re-read screen */
#define ENG_SH_WT      0x5457   /* Write a screen */
#define ENG_SH_RW      0x5752   /* rewrite func */
#define ENG_SH_WM      0x4d57   /* Write Message line */
#define ENG_SH_AL      0x4c41   /* Write Message line */
#define ENG_SH_WL      0x4c57   /* Write to log only */
#define ENG_SH_CM      0x4d43   /* Clear Message line */
#define ENG_SH_DS      0x5344   /* Detach screen */
#define ENG_SH_CN      0x434e   /* Connect webui screen */
#define ENG_SH_KL      0x4c4b   /* Terminate engine */
#define ENG_SH_SN      0x4e53   /* Tell web ui to look for son */
#define ENG_SH_GF      0x4647   /* Get font */
#define ENG_SH_GS      0x5347   /* Get font size */
#define ENG_SH_GR      0x5247   /* Get row pixel size */
#define ENG_SH_GC      0x4347   /* Get col pixel size */
#define ENG_SH_SF      0x4653   /* Set font */
#define ENG_SH_SS      0x5353   /* Set font size */
#define ENG_SH_SR      0x5253   /* Set row pixel size */
#define ENG_SH_SC      0x4353   /* Set col pixel size */
#define ENG_SH_LI      0x494c   /* Login */

#define ENG_SH_ST      0x5453   /* Get status of sh */
#define ENG_SH_EN      0x4e45   /* End sh */
#define ENG_SH_IN      0x4e49   /* Init SH */

/* not implmented */
#define ENG_SH_SZ      0x5a53   /* Resize the screen */
#define ENG_SH_RS      0x5352   /* Reset the screen */
#define ENG_SH_BL      0x4c42   /* Bell */
#define ENG_SH_TD      0x4454   /* Task Done */
#define ENG_SH_AR      0x5241   /* Attrib Reset */
#define ENG_SH_AS      0x5341   /* Attrib Save */
#define ENG_SH_AT      0x5441   /* Attrib Set */
#define ENG_SH_AC      0x4341   /* Attrib Clear */

#define ENG_SH_OK      0x2a2a   /* Good status */
#define ENG_SH_NC      0x434e   /* ncurses error */
#define ENG_SH_ER      0x5245   /* general error */

/*
  Define layout of standard top line
*/
#define ENG_SH_L1MAPN  8                 /* dspl to screen name field */
#define ENG_SH_L1STEP  ENG_SH_L1MAPN+8   /* dspl to step field */
#define ENG_SH_L1IKEY  ENG_SH_L1STEP+4   /* dspl to input key field */
#define ENG_SH_L1DEVN  ENG_SH_L1IKEY+2   /* dspl to device name field */
#define ENG_SH_L1TIME  ENG_SH_L1DEVN+2   /* dspl to time field */
#define ENG_SH_L1DATE  ENG_SH_L1TIME+8   /* dspl to date field */
#define ENG_SH_L1CURS  ENG_SH_L1DATE+8   /* dspl to cursor position field */
#define ENG_SH_L1LNTH  ENG_SH_L1CURS+4   /* total length of top line data */



struct _sh_parm
{
  char            id[12];       /* task id */
  char            screen[8];    /* Screen name */
  union rint      function;     /* function */
  union rint      status;       /* status */
  union rlng      file;         /* template file name */
  short int       length;       /* length of data area */
  char           *data;         /* pointer to user's data */
  int             row;          /* return row */
  int             col;          /* return col */
};


