/*
*  C Implementation: sh102
*
* Description: Write to the screen
*
*
* Author: Kevin Lloyd <kalloyd@gmail.com>, (C) 2005
*
* Copyright: See COPYING file that comes with this distribution
*
*/
int
sh102 ()
{
  short int       plin, pcol, plen, ulen, urln, ulin, ucol;   /* unpro length,line,colum */
  short int       i;
  int             datinx, rtn_status;
  char            *converted_data;
  FIELD           *field;
  short int       active_num;
  struct _sh_field_info *extra_field_info;             /* utyp,tcap,wcap */

  eng_debug ("sh102: Write screen: %8.8s", Sh_sh->data + ENG_SH_L1MAPN);

  /* lock screen for processing */
  sh206 ();
  /* Time stamp */
  sh210 ();

  Sh_def_background = Sh_def_sys_background;
  Sh_def_protected = Sh_def_sys_protected;
  Sh_def_unprotected = Sh_def_sys_unprotected;

  /* If screen name changed then release old and build new */
  if (memcmp (Sh_prev_screen, Sh_sh->data + ENG_SH_L1MAPN, 8) != 0)
    {
      memcpy (Sh_prev_screen, Sh_sh->data + ENG_SH_L1MAPN, 8);      /* Save Screen name */

      if (sh203 () != 0)                                            /* Is screen there */
        return (-1);

      Sh_fh.hd.current_id.nl = ENG_FH_4STARS;

      /* Read TEMP */
      if (sh211 () != 0)
        return (-1);

      /* See if we have a header record */
      plen = cnvbin (Sh_temp02->tmplen, 2);
      ulen = cnvbin (Sh_temp02->tmulen, 2);

      /* header record? */
      if ((!plen) && (!ulen))
        {
          /* Point to the header info in the temp data area */
          Sh_temp_header = (struct _tmhdr *) Sh_temp02->tmpdat;

          /* See if this is a popup */
          if ((rtn_status = sh204 ()) == 0)
          {
            /* Valid popup, create a new window */
            sh503 (1);

            Sh_scr_ptr->current_win = newwin (cnvbin (Sh_temp_header->hdlins,2)+1, cnvbin (Sh_temp_header->hdcols,2)+1, cnvbin (Sh_temp_header->hdtlin,2) - 1, cnvbin (Sh_temp_header->hdlcol, 2) - 1);

            keypad (Sh_scr_ptr->current_win, TRUE);

            /* Set colors */
            if (sh611 (Sh_scr_ptr->current_win, Sh_def_protected) < 0)
              return (-1);

            /* Offset userdata beyond top line */
            Sh_scr_ptr->beg_usr_data = ENG_SH_L1LNTH;
            /* Offset passed first line */
            Sh_scr_ptr->line_adj = 1;
          }
          else if (rtn_status == 1)
          {
            eng_error_out ("sh102 - Invalid popup parametors");
            sh999 (ENG_SH_ER);
          }
          else if (rtn_status == -1)
          {
            sh999 (ENG_SH_EN);
          }
          else
          {
            /* Was NOT a popup so just reinit current window */
            sh503 (0);
          }

          /* Save a copy of the header record */
          memcpy (&Sh_scr_ptr->temp_header, (char *) Sh_temp_header, sizeof (struct _tmhdr));

          /* Background if non-null, and non-blank */
          if ((Sh_temp_header->hdbatr != '\0') && (Sh_temp_header->hdbatr != ' '))
            Sh_def_background = Sh_temp_header->hdbatr;

          /* Protected if non-null, and non-blank */
          if ((Sh_temp02->tmpatr != '\0') && (Sh_temp02->tmpatr != ' '))
            Sh_def_protected = Sh_temp02->tmpatr;

          /* Unprotected if non-null, and non-blank */
          if ((Sh_temp02->tmuatr != '\0') && (Sh_temp02->tmuatr != ' '))
            Sh_def_unprotected = Sh_temp02->tmuatr;

          if (Sh_temp_header->hdlin1 == 'N')                     /* Hide line 1 */
          {
            Sh_scr_ptr->line_adj = 1;
            Sh_scr_ptr->beg_usr_data = ENG_SH_L1LNTH;           /* Offset userdata beyond top line */
          }
        }
      else
        {
          /* Reinit the current window struct */
          sh503 (0);

          /* Re-read first line */
          Sh_fh.hd.current_id.nl = ENG_FH_4STARS;
          memset (&Sh_scr_ptr->temp_header, ' ', sizeof (struct _tmhdr));
        }

      if (Sh_flags.hide_top_line && (memcmp (Sh_prev_screen, "MENU", 4) != 0) && (memcmp (Sh_prev_screen, "SYSM", 4) != 0))
        {
          Sh_scr_ptr->line_adj = 1;
          Sh_scr_ptr->beg_usr_data = ENG_SH_L1LNTH;
        }

        /* Get cursor position */
      sh212 ();
      /* Index of active field */
      active_num = 0;

      datinx = Sh_scr_ptr->beg_usr_data;
      /* Save line 1 data */
      memcpy (Sh_scr_ptr->line1_data, Sh_sh->data, ENG_SH_L1LNTH);

      /* Read each line in TEMP table */
      while (Sh_fh.hd.next_id.nl != ENG_FH_4STARS)
        {
          /* Read TEMP */
          if (sh211 () != 0)
            return (-1);

          /* In curses cordinates start at 0, but in the engine
             screens are build based on starting from 1 */
          plin = cnvbin (Sh_temp02->tmplin, 2) - 1;
          pcol = cnvbin (Sh_temp02->tmpcol, 2) - 1;
          plen = cnvbin (Sh_temp02->tmplen, 2);
          ulin = cnvbin (Sh_temp02->tmulin, 2) - 1;
          ucol = cnvbin (Sh_temp02->tmucol, 2) - 1;
          ulen = cnvbin (Sh_temp02->tmulen, 2);
          if (isdigit(*Sh_temp02->tmurln))
            urln = cnvbin (Sh_temp02->tmurln, 3);
          else
            urln = ulen;

          /* If we are skipping top line adjust lines down further */
          ulin -= Sh_scr_ptr->line_adj;
          plin -= Sh_scr_ptr->line_adj;

          if (plin >= 0 && plen > 0)
            {
              eng_debug ("sh102: Processing protected data");

              /* Save protected data for later */

              /* First see what the attrib for the field is */
              if ((Sh_temp02->tmpatr >= COLOR_PRO_BEG && Sh_temp02->tmpatr <= COLOR_PRO_END) || (Sh_temp02->tmpatr >= COLOR_UPRO_BEG && Sh_temp02->tmpatr <= COLOR_UPRO_END) || Sh_temp02->tmpatr == NONDISP_PRO || Sh_temp02->tmpatr == NONDISP_UPRO || Sh_temp02->tmpatr == DEFAULT_PRO)
                Sh_scr_ptr->sav_pdat->tmpatr = Sh_temp02->tmpatr;
              else
                Sh_scr_ptr->sav_pdat->tmpatr = DEFAULT_PRO;   /* Default to protected */

              /* Protected and hidden.. no need to even save */
              if (Sh_scr_ptr->sav_pdat->tmpatr != NONDISP_PRO && Sh_scr_ptr->sav_pdat->tmpatr != NONDISP_UPRO)
                {
                  Sh_scr_ptr->sav_pdat->tmpcol = pcol;
                  Sh_scr_ptr->sav_pdat->tmplin = plin;

                  Sh_scr_ptr->sav_pdat->tmplen = plen;
                  memcpy (Sh_scr_ptr->sav_pdat->tmpdat, Sh_temp02->tmpdat, 20);

                  Sh_scr_ptr->sav_pdat->tmpfnt = Sh_temp02->tmpfnt;
                  Sh_scr_ptr->sav_pdat->tmpsty = Sh_temp02->tmpsty;
                  snprintf (Sh_scr_ptr->sav_pdat->tmpfsz, 3, "%s", Sh_temp02->tmpfsz);

                  if (!Sh_scr_ptr->sav_pdat->tmpnxt)              /* Create next chain in struct */
                    {
                      Sh_scr_ptr->sav_pdat->tmpnxt = (struct _savp *) eng_malloc (sizeof (struct _savp), "sh102:Sh_scr_ptr->sav_pdat->tmpnxt");
                      Sh_scr_ptr->sav_pdat = Sh_scr_ptr->sav_pdat->tmpnxt;
                      Sh_scr_ptr->sav_pdat->tmpnxt = NULL;
                    }
                  else
                    Sh_scr_ptr->sav_pdat = Sh_scr_ptr->sav_pdat->tmpnxt;
                }
            }

          if (ulin >= 0 && ulen > 0)
            {
              /* Create curses form */
              eng_debug ("sh102: Processing unprotected data: ucol: %d ulin: %d ulen: %d num_field: %d", ucol, ulin, ulen, Sh_scr_ptr->num_field);

              /* Create field */
              if ((Sh_scr_ptr->field[Sh_scr_ptr->num_field] = new_field (1, ulen, ulin, ucol, 0, 0)) == NULL)
                return(sh998("sh102: new_field",ENG_SH_NC));

              field = Sh_scr_ptr->field[Sh_scr_ptr->num_field];

              set_field_just (field, NO_JUSTIFICATION);
              field_opts_off (field, O_BLANK);
              field_opts_off (field, O_AUTOSKIP);
              field_opts_off (field, O_STATIC);
              set_max_field(field, urln);

              extra_field_info = (struct _sh_field_info *) eng_malloc (sizeof (struct _sh_field_info), "sh102: extra_field_info");

              extra_field_info->utyp = Sh_temp02->tmutyp;
              extra_field_info->ufnt = Sh_temp02->tmufnt;
              extra_field_info->usty = Sh_temp02->tmusty;
              strncpy (extra_field_info->ufsz, make_string (Sh_temp02->tmufsz,sizeof (Sh_temp02->tmufsz),1,0,0,0), sizeof (extra_field_info->ufsz));
              strncpy (extra_field_info->tcap, make_string (Sh_temp02->tmtcap,sizeof (Sh_temp02->tmtcap),1,0,0,0), sizeof (extra_field_info->tcap));
              strncpy (extra_field_info->wcap, make_string (Sh_temp02->tmwcap,sizeof (Sh_temp02->tmwcap),1,0,0,0), sizeof (extra_field_info->wcap));
              strncpy (extra_field_info->bcap, make_string (Sh_temp02->tmbcap,sizeof (Sh_temp02->tmbcap),1,0,0,0), sizeof (extra_field_info->bcap));
              strncpy (extra_field_info->bval, make_string (Sh_temp02->tmbval,sizeof (Sh_temp02->tmbval),1,0,0,0), sizeof (extra_field_info->bval));
              strncpy (extra_field_info->bkey, make_string (Sh_temp02->tmbkey,sizeof (Sh_temp02->tmbkey),1,0,0,0), sizeof (extra_field_info->bkey));
              extra_field_info->cur_col = 0;
              extra_field_info->org_att = Sh_temp02->tmuatr;
              set_field_userptr(field, (void *) extra_field_info);

              if (Sh_flags.filter_pad)
                converted_data = make_string (Sh_sh->data + datinx, urln, 3, Sh_def_pad_char, 1, 0);
              else
                converted_data = make_string (Sh_sh->data + datinx, urln,0,0,1,0);

              /* Give the field data */
              if (set_field_buffer (field, 0, converted_data) != E_OK)
                return (sh998("sh102: set_field_buffer",ENG_SH_NC));

              /* Process the attribute */
              sh600 (field, Sh_temp02->tmuatr);

              /* Unprotected active data */
              if (SH_ACTIVE (field))
                active_num++;

              /* Negitive row indicates we want a certain numbered field */
              if (Sh_scr_ptr->cur_row < 0)
                {
                  /* Column has the number of the field they want */
                  if (active_num == Sh_scr_ptr->cur_col)
                    {
                      Sh_scr_ptr->cur_row = ulin;
                      Sh_scr_ptr->cur_col = ucol;
                      Sh_scr_ptr->focus_field = Sh_scr_ptr->num_field;
                    }
                }
              else
                {
                  if ((Sh_scr_ptr->cur_row == ulin) && (Sh_scr_ptr->cur_col >= ucol && Sh_scr_ptr->cur_col <= ucol + ulen))
                    Sh_scr_ptr->focus_field = Sh_scr_ptr->num_field;
                }

              datinx += urln;
              Sh_scr_ptr->num_field++;
              Sh_scr_ptr->field[Sh_scr_ptr->num_field] = 0;      /* Indicate last field */
            }
        }

      /* If we did not find a field set to 0 */
      if (Sh_scr_ptr->cur_row < 0)
         Sh_scr_ptr->cur_row = Sh_scr_ptr->cur_col = Sh_scr_ptr->focus_field = 0;

      Sh_scr_ptr->sav_pdat->tmplen = 0;                          /* Mark end of protected data */
      if (sh610 (Sh_scr_ptr->current_win) != 0)                  /* Output data to screen */
        return (sh999 (ENG_SH_EN));
    }
  else
    {
      /* Same screen as previous just output unprotected */
      /* Get cursor position */
      sh212 ();
      /* index of active fields */
      active_num = 0;

      datinx = Sh_scr_ptr->beg_usr_data;
      memcpy (Sh_scr_ptr->line1_data, Sh_sh->data, ENG_SH_L1LNTH);      /* Save line 1 data */

      for (i = 0; i < Sh_scr_ptr->num_field; i++)
        {
          field = Sh_scr_ptr->field[i];

          if (Sh_flags.filter_pad)
            converted_data = make_string (Sh_sh->data + datinx, field->maxgrow, 3, Sh_def_pad_char, 1, 0);
          else
            converted_data = make_string (Sh_sh->data + datinx, field->maxgrow,0,0,1,0);

          if (set_field_buffer (field, 0, converted_data) != E_OK)    /* Give the field data */
            return (sh998("sh102: set_field_buffer",ENG_SH_NC));
          eng_debug ("sh102: Set buffer<%s> length: %d", field_buffer (field, 0), field->maxgrow);

          if (SH_ACTIVE (field))
            active_num++;

          /* Negitive row indicates we want a certain numbered field */
          if (Sh_scr_ptr->cur_row < 0)
            {
              /* Column has the number of the field they want */
              if (active_num == Sh_scr_ptr->cur_col)
                {
                  Sh_scr_ptr->cur_row = field->frow;
                  Sh_scr_ptr->cur_col = field->fcol;
                  Sh_scr_ptr->focus_field = i;
                  set_current_field (Sh_scr_ptr->form, field);
                }
            }
          else
            {
              if ((Sh_scr_ptr->cur_row == field->frow) && (Sh_scr_ptr->cur_col >= field->fcol && Sh_scr_ptr->cur_col <= field->fcol + field->cols))
                {
                  Sh_scr_ptr->focus_field = i;
                  set_current_field (Sh_scr_ptr->form, field);
                }
            }
          datinx += field->maxgrow;
        }

      /* If we did not find a field set to 0 */
      if (Sh_scr_ptr->cur_row < 0)
        Sh_scr_ptr->cur_row = Sh_scr_ptr->cur_col = Sh_scr_ptr->focus_field = 0;

      if (Sh_flags.screen_mode == SH_WEB)
        eng_post_form (Sh_scr_ptr->form);
    }

  sh609 (Sh_scr_ptr->current_win, Sh_scr_ptr->cur_row, Sh_scr_ptr->cur_col);     /* Move curser */

  eng_wrefresh (Sh_scr_ptr->current_win);                                       /* Flush output */

  sh208 ();                                                                     /* release for others */

  return (0);
}

