/*
*  C Implementation: sh107
*
* Description: Write a message to the screen
* mode 0=clear, 1=write, 2=log only
*
* Author: Kevin Lloyd <kalloyd@gmail.com>, (C) 2005
*
* Copyright: See COPYING file that comes with this distribution
*
*/

void
sh107 (int mode)
{
  int             msglth;

  eng_debug ("sh107: Write message to message lines");

  sh109 ();                                       /* Write to log file */

  msglth = Sh_sh->length;                         /* get length from user v2.1 */
  if (msglth > SH_TMESG_LEN)                      /* make sure it will fit */
    msglth = SH_TMESG_LEN;

  /* if writting new message */
  if (mode)
    {
      if (memcmppn (Sh_mesg2, ' ', SH_TMESG_LEN) != 0)
        memcpy (Sh_mesg1, Sh_mesg2, SH_TMESG_LEN);

      /* clear bottom message */
      memset (Sh_mesg2, ' ', SH_TMESG_LEN);
      memcpy (Sh_mesg2, Sh_sh->data, msglth);
    }
  else
    {
      /* not writing, simply clear */
      memset (Sh_mesg1, ' ', SH_TMESG_LEN);
      memset (Sh_mesg2, ' ', SH_TMESG_LEN);
    }

  sh502 (0);                                              /* display message lines */
}

/*
  sh107c() - WRITE MESSAGE FROM CHILD
 
  This is the asynchronous routine triggered by signal (SIGUSR1) from
  child. This is done whenever child has placed new message in our
  message file and wants us to write it for him. This method is used
  because only parent can properly control all aspects of the user's
  terminal.
*/

void
sh107c ()
{
  int             msglth;
  struct _mesg_buf
    {
      long     mtype;                    /* message type, must be > 0 */
      char     mtext[SH_TMESG_LEN];      /* message data */
    }
  mesg_buf;

  eng_debug ("sh107c: Write message to message lines via caught signal signal");

  if (memcmp (Sh_mesg2, "  ", 2) != 0)
    memcpy (Sh_mesg1, Sh_mesg2, SH_TMESG_LEN);

  memset (Sh_mesg2, ' ', SH_TMESG_LEN);            /* clear bottom message */

  /* recieve any message */
  mesg_buf.mtype = 1;
  while ((msglth = msgrcv (Sh_mesg_id, &mesg_buf, SH_TMESG_LEN, 0L, IPC_NOWAIT + MSG_NOERROR)) > 0)
  {
    memcpy (Sh_mesg2, mesg_buf.mtext, SH_TMESG_LEN);
    sh502 (1);                                            /* display message lines as is */
  }
}

