/*
*  C Implementation: sh130
*
* Description: Process keys pressed from user
*
*
* Author: Kevin Lloyd <kalloyd@gmail.com>, (C) 2005
*
* Copyright: See COPYING file that comes with this distribution
*
*/

unsigned short  check_mapping (unsigned short, char *key);
unsigned short  check_special_keys (WINDOW *win);
void            clr_eos ();
void            clr_eol ();
void            clr_eof ();
void            next_field ();
void            prev_field ();
void            end_field ();
void            home_field ();
void            up_field ();
void            down_field ();
int             left_char ();
void            normal_field ();
void            highlight_field ();
void            select_field (int);
void            set_field_cursor (FORM *, int);
int             get_field_column (FORM *);
int             get_field_maxgrow (FIELD *);

unsigned int    sh_fired_key = 0;

int
sh130 (WINDOW *win, char mode)
{
  /* Local functions */

  void sh_info ();
  unsigned int    ch;
  unsigned int    last_ch = 0;
  unsigned int    frst_toggle,field_skip;
  unsigned int    save_col, i;

  eng_debug ("sh130: Curses key process, current row: %d colum: %d", Sh_scr_ptr->cur_row, Sh_scr_ptr->cur_col);

  /* Make sure we have a current form to work with */
  if (Sh_scr_ptr->form)
    {
      /* Make sure were in the right edit mode */
      if (Sh_ins_toggle)
        {
          form_driver (Sh_scr_ptr->form, REQ_INS_MODE);
          curs_set (1);
        }
      else
        {
          form_driver (Sh_scr_ptr->form, REQ_OVL_MODE);
          curs_set (2);
        }

      if ((memcmp (Sh_prev_screen, "MENU", 4) != 0) && (memcmp (Sh_prev_screen, "LEVL", 4) != 0) && Sh_scr_ptr->form->current->fcol == Sh_scr_ptr->cur_col)
        {
          select_field (0);
          frst_toggle = 1;
        }
      else
        {
          highlight_field ();
          check_utype ();
          frst_toggle = 0;
        }
    }
  wrefresh (win);
  Sh_key_pressed.ui = 0;

  ch = '\0';
  while (!Sh_key_pressed.ui)
    {
      if (!sh_fired_key)
        {
          /* I think CDK takes us out of raw so put it back here */
          raw ();

          /* Set a 1 second timeout on read so we can check for marques */
          wtimeout (win, 200);
          while ((ch = wgetch (win)) == ERR)
            {
              /* See if there are any marque fields */
              if (!marquee_fields (win))
                {
                  /* No marques, Set read back to blocking */
                  wtimeout (win, -1);
                }
            }
          /* Set read back to blocking */
          wtimeout (win, -1);
        }
      else
        {
          /* fired_key is a key that was generated somewhere else in a function
             thats needs to be handled here */
          ch = sh_fired_key;
          sh_fired_key = 0;
        }

      eng_debug ("sh130: Key pressed Hex(%x) Char(%c)", ch, ch);
      switch (ch)
        {
        case '\x05':                                      /* Ctrl E E EXIT */
          {
            if (last_ch == '\x05')
              {
                sh999(ENG_SH_EN);
                return (-1);
              }
            last_ch = ch;
            break;
          }

        case '\x03':                                      /* Ctrl C Clear mesg lines */
          {
            /* We loose cursor position after clearing mesg line */
            save_col = get_field_column (Sh_scr_ptr->form);
            sh107 (0);
            for (i=get_field_column (Sh_scr_ptr->form); i > save_col; i--)
              form_driver (Sh_scr_ptr->form, REQ_PREV_CHAR);
            break;
          }

        case '\x12':                                      /* Ctrl R Redraw screen */
          {
            sh111 ();
            break;
          }

        case '\x11':                                      /* Ctrl Q Info popup */
          {
            sh_info ();
            break;
          }

        case KEY_F(1):
            {
              Sh_key_pressed.ui = check_mapping (F1, F1_map);
              break;
            }
        case KEY_F(2):
            {
              Sh_key_pressed.ui = check_mapping (F2, F2_map);
              break;
            }
        case KEY_F(3):
            {
              Sh_key_pressed.ui = check_mapping (F3, F3_map);
              break;
            }
        case KEY_F(4):
            {
              Sh_key_pressed.ui = check_mapping (F4, F4_map);
              break;
            }
        case KEY_F(5):
            {
              Sh_key_pressed.ui = check_mapping (F5, F5_map);
              break;
            }
        case KEY_F(6):
            {
              Sh_key_pressed.ui = check_mapping (F6, F6_map);
              break;
            }
        case KEY_F(7):
            {
              Sh_key_pressed.ui = check_mapping (F7, F7_map);
              break;
            }
        case KEY_F(8):
            {
              Sh_key_pressed.ui = check_mapping (F8, F8_map);
              break;
            }
        case KEY_F(9):
            {
              Sh_key_pressed.ui = check_mapping (F9, F9_map);
              break;
            }
        case KEY_F(10):
            {
              Sh_key_pressed.ui = check_mapping (F10, F10_map);
              break;
            }
        case KEY_F(11):
            {
              Sh_key_pressed.ui = check_mapping (F11, F11_map);
              break;
            }
        case KEY_F(12):
            {
              Sh_key_pressed.ui = check_mapping (F12, F12_map);
              break;
            }

        /* Shifted F1-F12 */
        case KEY_F(13):
            {
              Sh_key_pressed.ui = check_mapping (F13, F13_map);
              break;
            }
        case KEY_F(14):
            {
              Sh_key_pressed.ui = check_mapping (F14, F14_map);
              break;
            }
        case KEY_F(15):
            {
              Sh_key_pressed.ui = check_mapping (F15, F15_map);
              break;
            }
        case KEY_F(16):
            {
              Sh_key_pressed.ui = check_mapping (F16, F16_map);
              break;
            }
        case KEY_F(17):
            {
              Sh_key_pressed.ui = check_mapping (F17, F17_map);
              break;
            }
        case KEY_F(18):
            {
              Sh_key_pressed.ui = check_mapping (F18, F18_map);
              break;
            }
        case KEY_F(19):
            {
              Sh_key_pressed.ui = check_mapping (F19, F19_map);
              break;
            }
        case KEY_F(20):
            {
              Sh_key_pressed.ui = check_mapping (F20, F20_map);
              break;
            }
        case KEY_F(21):
            {
              Sh_key_pressed.ui = check_mapping (F21, F21_map);
              break;
            }
        case KEY_F(22):
            {
              Sh_key_pressed.ui = check_mapping (F22, F22_map);
              break;
            }
        case KEY_F(23):
            {
              Sh_key_pressed.ui = check_mapping (F23, F23_map);
              break;
            }
        case KEY_F(24):
            {
              Sh_key_pressed.ui = check_mapping (F24, F24_map);
              break;
            }

        case KEY_ENTER:
        case '\n':
          {
            Sh_key_pressed.ui = check_mapping (ENG_SH_EN, ENT_map);
            break;
          }

        case '\x1b':                                   /* ESC */
          {
            /* See if a special key sequence was sent */
            if (!check_special_keys (win))
              Sh_key_pressed.ui = check_mapping (ENG_SH_AT, ESC_map);
            break;
          }

        case KEY_HOME:
          {
            if (check_mapping (1, HOME_map) == 1)
              home_field ();
            frst_toggle = 1;
            break;
          }

        case KEY_END:
          {
            if (check_mapping (1, END_map) == 1)
              end_field ();
            frst_toggle = 1;
            break;
          }

        case KEY_DOWN:
          {
            if (check_mapping (1, DOWN_map) == 1)
              down_field ();
            frst_toggle = 1;
            break;
          }

        case KEY_UP:
          {
            if (check_mapping (1, UP_map) == 1)
              up_field ();
            frst_toggle = 1;
            break;
          }

        case KEY_LEFT:
          {
            if (frst_toggle)
              {
                highlight_field ();
                form_driver (Sh_scr_ptr->form, REQ_BEG_FIELD);
                frst_toggle = 0;
              }
            else
              {
                frst_toggle = left_char ();
              }
            break;
          }

        case KEY_RIGHT:
          {
            if (frst_toggle)
              {
                highlight_field ();
                form_driver (Sh_scr_ptr->form, REQ_END_FIELD);
                frst_toggle = 0;
              }
            else
              {
                if (form_driver (Sh_scr_ptr->form, REQ_NEXT_CHAR) == E_REQUEST_DENIED)
                  {
                    next_field ();
                    frst_toggle = 1;
                  }
              }
            break;
          }

        case KEY_BACKSPACE:
        case '\x7f':
          {
            highlight_field ();
            if (check_mapping (1, BACKSPACE_map) == 1)
              {
                if (frst_toggle && Sh_flags.first_key_blank)
                  {
                    form_driver (Sh_scr_ptr->form, REQ_CLR_FIELD);
                    form_driver (Sh_scr_ptr->form, REQ_BEG_FIELD);
                  }
                frst_toggle = 0;
                if (form_driver (Sh_scr_ptr->form, REQ_PREV_CHAR) == E_REQUEST_DENIED)
                  {
                    normal_field ();
                    form_driver (Sh_scr_ptr->form, REQ_PREV_FIELD);
                    form_driver (Sh_scr_ptr->form, REQ_END_FIELD);
                    highlight_field ();
                  }
                form_driver (Sh_scr_ptr->form, REQ_DEL_CHAR);
              }
            break;
          }

        case KEY_DC:                                      /* Delete */
          {
            highlight_field ();
            if (frst_toggle && Sh_flags.first_key_blank)
              {
                form_driver (Sh_scr_ptr->form, REQ_CLR_FIELD);
                form_driver (Sh_scr_ptr->form, REQ_BEG_FIELD);
              }
            frst_toggle = 0;
            form_driver (Sh_scr_ptr->form, REQ_DEL_CHAR);
            break;
          }

        case KEY_IC:                                      /* Insert */
          {
            highlight_field ();
            if (Sh_ins_toggle)
              {
                form_driver (Sh_scr_ptr->form, REQ_OVL_MODE);
                curs_set (2);
                Sh_ins_toggle = OFF;
              }
            else
              {
                form_driver (Sh_scr_ptr->form, REQ_INS_MODE);
                curs_set (1);
                Sh_ins_toggle = ON;
              }
            frst_toggle = 0;
            break;
          }


        case KEY_NPAGE:                                   /* Page Down */
          {
            if (check_mapping (1, PGDWN_map) == 1)
              clr_eof ();
            frst_toggle = 1;
            break;
          }

        case KEY_PPAGE:                                   /* Page Up */
          {
            if (check_mapping (1, PGUP_map) == 1)
              next_field ();
            frst_toggle = 1;
            break;
          }

        case KEY_BTAB:                                    /* Back tab */
          {
            if (form_driver (Sh_scr_ptr->form, REQ_PREV_CHAR) == E_REQUEST_DENIED || frst_toggle == 1)        /* Test for beg of field */
              {
                prev_field ();
                frst_toggle = 1;
              }
            else
              {
                highlight_field ();
                form_driver (Sh_scr_ptr->form, REQ_BEG_FIELD);
              }
            break;
          }

        case '\t':                                       /* Tab */
          {
            if (check_mapping (1, TAB_map) == 1)
              next_field ();
            frst_toggle = 1;
            break;
          }

        default:
          {
            /* If this is a normal character, it gets displayed */
            highlight_field ();
            if (frst_toggle && Sh_flags.first_key_blank)
              {
                form_driver (Sh_scr_ptr->form, REQ_CLR_FIELD);
                form_driver (Sh_scr_ptr->form, REQ_BEG_FIELD);
              }
            frst_toggle = 0;

            /* See if after this character we need to advance to the next field */
            if (get_field_column (Sh_scr_ptr->form) == (get_field_maxgrow (current_field (Sh_scr_ptr->form)) - 1))
              field_skip = ON;
            else
              field_skip = OFF;

            if (form_driver (Sh_scr_ptr->form, ch) == E_REQUEST_DENIED)
              {
                /* Char did not fit so where going to run the last char off the screen */
                save_col = get_field_column (Sh_scr_ptr->form);
                form_driver (Sh_scr_ptr->form, REQ_END_FIELD);
                form_driver (Sh_scr_ptr->form, REQ_DEL_CHAR);
                set_field_cursor (Sh_scr_ptr->form, save_col);

                /* Try to add char again */
                form_driver (Sh_scr_ptr->form, ch);
              }

            if (field_skip == ON)
              {
                wrefresh (win);                  /* So char's will show before advancing */
                next_field ();
                frst_toggle = 1;
              }
            break;
          }
        } // switch

      wrefresh (win);
      if (Sh_key_pressed.ui == 0 && mode == 1)
        {
          *Sh_key_pressed.ci = ch;
        }
    }  // while

  normal_field ();
  return (0);
}

void set_field_cursor (FORM *form, int cursor)
{
  int i;

  if (form)
    {
      i = form->curcol;
      while (i > cursor || i < cursor)
        {
          if (i > cursor)
            {
              form_driver (form, REQ_PREV_CHAR);
              i--;
            }
          else
            {
              form_driver (form, REQ_NEXT_CHAR);
              i++;
            }
        }
    }
}

int get_field_column (FORM *form)
{
  if (form)
    return (form->curcol);
  else
    return (0);
}

int get_field_maxgrow (FIELD *field)
{
  if (field)
    return (field->maxgrow);
  else
    return (0);
}

unsigned short
check_special_keys (WINDOW *win)
{
  int   i = 0;
  char  sequence[10];
  char  ch;
  int   xpos, ypos;
  char  return_val = 0;

  char  *KEY_LEFT_BOX   = "O5D";
  char  *KEY_RIGHT_BOX  = "O5C";
  char  *KEY_UP_BOX     = "O5A";
  char  *KEY_DOWN_BOX   = "O5B";

  eng_debug ("check_special_keys - Check for special keys starting with ESC");

  memset (sequence, '\0', sizeof (sequence));

  /* This should capture only the special keys (hopefully)*/
  wtimeout (win, 0);
  while ((ch = wgetch (win)) != ERR && i < 10)
    {
      *(sequence+i++) = ch;
    }

  /* Reset back to blocking */
  wtimeout (win, -1);

  eng_debug ("check_special_keys: sequence: (%s)", sequence);

  if (i > 0)
    {
      /* Check for box moving keys */
      /* Window must be a popup */
      if (Sh_scr_ptr->is_popup)
        {
          if (strcmp (sequence, KEY_LEFT_BOX) == 0)
            {
              getbegyx (win, ypos, xpos);
              mvwin (win, ypos, xpos - 2);
              return_val = 1;
            }
          else if (strcmp (sequence, KEY_RIGHT_BOX) == 0)
            {
              getbegyx (win, ypos, xpos);
              mvwin (win, ypos, xpos + 2);
              return_val = 1;
            }
          else if (strcmp (sequence, KEY_UP_BOX) == 0)
            {
              getbegyx (win, ypos, xpos);
              mvwin (win, ypos - 1, xpos);
              return_val = 1;
            }
          else if (strcmp (sequence, KEY_DOWN_BOX) == 0)
            {
              getbegyx (win, ypos, xpos);
              mvwin (win, ypos + 1, xpos);
              return_val = 1;
            }

          /* If we've moved the popup we must refresh
          the bottom screens cuz it is now corrupted */
          for (i=0; i < Sh_scr_data[Sh_current_scr_core].cur_window; i++)
            {
              touchwin (Sh_scr_data[Sh_current_scr_core].window[i]->current_win);
              wrefresh (Sh_scr_data[Sh_current_scr_core].window[i]->current_win);
            }
          wrefresh (win);
        }

      /* If we didn't find a special key put the keys
      back into the buffer */
      if (!return_val)
        {
          for (i=0; i < strlen(sequence); i++)
            {
              ungetch (*(sequence+i));
            }
        }
    }

  return (return_val);
}

unsigned short
check_mapping (unsigned short pressed_key, char *key)
{
  union rint rtn_key;

  rtn_key.ui = pressed_key;
  Sh_config_keys = Sh_beg_config_keys;
  while (Sh_config_keys)
    {
      if (memcmp (Sh_config_keys->config_keys->key, key, strlen (key) + 1) == 0)
        {
          if (memcmp (Sh_config_keys->config_keys->new_key, "KEY_", 3) == 0)
            {
              eng_debug ("check_mapping: remapped key to: %c%c", *(Sh_config_keys->config_keys->new_key + 4), *(Sh_config_keys->config_keys->new_key + 5));
              memcpy (rtn_key.ci, Sh_config_keys->config_keys->new_key + 4, 2);
              break;
            }
          else
            {
              if (memcmp (Sh_config_keys->config_keys->new_key, "REQ_CLR_EOS", 11) == 0)
                {
                  clr_eos ();
                  rtn_key.ui = 0;
                }
              else if (memcmp (Sh_config_keys->config_keys->new_key, "REQ_CLR_EOL", 11) == 0)
                {
                  clr_eol ();
                  rtn_key.ui = 0;
                }
              else if (memcmp (Sh_config_keys->config_keys->new_key, "REQ_CLR_EOF", 11) == 0)
                {
                  clr_eof ();
                  rtn_key.ui = 0;
                }
              else if (memcmp (Sh_config_keys->config_keys->new_key, "REQ_NEXT_FIELD", 14) == 0)
                {
                  next_field ();
                  rtn_key.ui = 0;
                }
              else if (memcmp (Sh_config_keys->config_keys->new_key, "REQ_PREV_FIELD", 14) == 0)
                {
                  prev_field ();
                  rtn_key.ui = 0;
                }
              else if (memcmp (Sh_config_keys->config_keys->new_key, "REQ_FIRST_FIELD", 15) == 0)
                {
                  home_field ();
                  rtn_key.ui = 0;
                }
              else if (memcmp (Sh_config_keys->config_keys->new_key, "REQ_DOWN_FIELD", 14) == 0)
                {
                  down_field ();
                  rtn_key.ui = 0;
                }
              else if (memcmp (Sh_config_keys->config_keys->new_key, "REQ_LEFT_CHAR", 13) == 0)
                {
                  left_char ();
                  rtn_key.ui = 0;
                }
              break;
            }
        }
      Sh_config_keys = Sh_config_keys->next_eng_config_keys_list;
    }
  return (rtn_key.ui);
}


void
clr_eos ()
{
  int             fheight, flen, ftop, fleft, foff, fbuf;
  int             current_row, current_col;

  if (Sh_scr_ptr->form)
    {
      normal_field ();
      field_info (current_field (Sh_scr_ptr->form), &fheight, &flen, &ftop, &fleft, &foff, &fbuf);
      current_row = ftop;
      current_col = fleft;
      do
        {
          form_driver (Sh_scr_ptr->form, REQ_CLR_EOL);
          form_driver (Sh_scr_ptr->form, REQ_NEXT_FIELD);
          field_info (current_field (Sh_scr_ptr->form), &fheight, &flen, &ftop, &fleft, &foff, &fbuf);
        }
      while (current_row <= ftop && !(current_row == ftop && current_col == fleft));
      select_field (0);
    }
}

void
clr_eol ()
{
  int             fheight, flen, ftop, fleft, foff, fbuf;
  int             current_row, current_col;

  if (Sh_scr_ptr->form)
    {
      normal_field ();
      field_info (current_field (Sh_scr_ptr->form), &fheight, &flen, &ftop, &fleft, &foff, &fbuf);
      current_row = ftop;
      current_col = fleft;
      do
        {
          form_driver (Sh_scr_ptr->form, REQ_CLR_EOL);
          form_driver (Sh_scr_ptr->form, REQ_NEXT_FIELD);
          field_info (current_field (Sh_scr_ptr->form), &fheight, &flen, &ftop, &fleft, &foff, &fbuf);
        }
      while (current_row == ftop && current_col != fleft);
      select_field (0);
    }
}

void
clr_eof ()
{
  if (Sh_scr_ptr->form)
    {
      normal_field ();
      form_driver (Sh_scr_ptr->form, REQ_CLR_EOF);
      form_driver (Sh_scr_ptr->form, REQ_NEXT_FIELD);
      select_field (0);
    }
}

int
left_char ()
{
  if (Sh_scr_ptr->form)
    {
      if (form_driver (Sh_scr_ptr->form, REQ_PREV_CHAR) == E_REQUEST_DENIED)
        {
          prev_field ();
          return (1);
        }
    }
  return (0);
}

void
next_field ()
{
  FIELD *field;

  if (Sh_scr_ptr->form)
    {
      normal_field ();
      field = Sh_scr_ptr->form->current;
      form_driver (Sh_scr_ptr->form, REQ_NEXT_FIELD);

      /* Because this can be fired from within other movement keys
        We have to not get select_field in a loop */
      if (field == Sh_scr_ptr->form->current)            /* If the field stayed the same do not check utype */
        select_field (1);
      else
        select_field (0);
    }
}

void
prev_field ()
{
  if (Sh_scr_ptr->form)
    {
      normal_field ();
      form_driver (Sh_scr_ptr->form, REQ_PREV_FIELD);
      select_field (0);
    }
}

void
home_field ()
{
  if (Sh_scr_ptr->form)
    {
      normal_field ();
      form_driver (Sh_scr_ptr->form, REQ_FIRST_FIELD);
      select_field (0);
    }
}

void
end_field ()
{
  int             fheight, flen, ftop, fleft, foff, fbuf;
  int             current_row, current_col;

  if (Sh_scr_ptr->form)
    {
      normal_field ();
      field_info (current_field (Sh_scr_ptr->form), &fheight, &flen, &ftop, &fleft, &foff, &fbuf);
      current_row = ftop;
      current_col = fleft;
      do
        {
          form_driver (Sh_scr_ptr->form, REQ_NEXT_FIELD);
          field_info (current_field (Sh_scr_ptr->form), &fheight, &flen, &ftop, &fleft, &foff, &fbuf);
        }
      while (current_row == ftop && current_col != fleft);
      select_field (0);
    }
}

void
down_field ()
{
  if (Sh_scr_ptr->form)
    {
      normal_field ();
      form_driver (Sh_scr_ptr->form, REQ_DOWN_FIELD);
      select_field (0);
    }
}

void
up_field ()
{
  if (Sh_scr_ptr->form)
    {
      normal_field ();
      form_driver (Sh_scr_ptr->form, REQ_UP_FIELD);
      select_field (0);
    }
}

void
normal_field ()
{
  FIELD *field;

  if (Sh_scr_ptr->form)
    {
      if (Sh_flags.pad_reverse)
        {
          field = Sh_scr_ptr->form->current;

          /* Remove '+' from padding when size of field is one */
          if (field->cols == 1)
            set_field_pad (field, ' ');

          /* Set the normal color */
          set_field_back (field, sh603 (SH_FORE_BLACK, SH_BACK_WHITE, SH_UNPRO_TYPE));
          set_field_fore (field, sh603 (SH_FORE_BLACK, SH_BACK_WHITE, SH_UNPRO_TYPE));

        }
    }
}

void
highlight_field ()
{
  if (Sh_scr_ptr->form)
    {
      if (Sh_flags.pad_reverse)
        {
          /* Remove '+' from padding when size of field is one */
          if (Sh_scr_ptr->form->current->cols == 1)
            set_field_pad (Sh_scr_ptr->form->current, ' ');
          set_field_back (Sh_scr_ptr->form->current, COLOR_PAIR (16));
          set_field_fore (Sh_scr_ptr->form->current, COLOR_PAIR (16));
          wrefresh (Sh_scr_ptr->current_win);
        }
    }
}

void
select_field (utype_mode)
{
  if (Sh_scr_ptr->form)
    {
      if (Sh_flags.pad_reverse)
        {
          set_field_back (Sh_scr_ptr->form->current, COLOR_PAIR (15));
          set_field_fore (Sh_scr_ptr->form->current, COLOR_PAIR (15));

          /* Paint field */
          form_driver (Sh_scr_ptr->form, REQ_BEG_FIELD);

          /* If field is only one char then we need something to
          show the cursor is there cuz on reverse video the cursor
          disapears */
          if (Sh_scr_ptr->form->current->cols == 1)
            set_field_pad (Sh_scr_ptr->form->current, '+');
          wrefresh (Sh_scr_ptr->current_win);
        }

      if (utype_mode == 0)
        {
          if (check_utype ())
            highlight_field ();
        }
    }
}

int check_utype ()
{
  struct _sh_field_info *extra_field_info;             /* utyp,tcap,wcap */
  int     return_val = 0;

  if (Sh_scr_ptr->form)
    {
      if ((extra_field_info = (struct _sh_field_info *) field_userptr(current_field (Sh_scr_ptr->form))))
        {
          if (extra_field_info->utyp == SH_TYPE_LIST)
            {
              if (item_list ((char *) extra_field_info))
                next_field ();
              return_val = 1;
            }
          else if (extra_field_info->utyp == SH_TYPE_FLST)
            {
              if (item_flist ((char *) extra_field_info))
                next_field ();
              return_val = 1;
            }
        }
    }
  return (return_val);
}

int
item_flist (char *extra_field_info_b)
{
  int             return_val = 0;
  FIELD          *field;
  struct _sh_field_info *extra_field_info;
  CDKSCREEN      *cdkscreen = 0;
  CDKSELECTION   *selection = 0;
  char          **list_array;
  char          **value_array;
  char *choices[] = {" ", "*"};
  int             item_count = 0, current_item, i, choice;
  int             box_width, box_height;
  int             default_item = 0;
  char            list_disp[3];
  char           *list_buffer;
  int             list_fd,list_rd_len;
  struct stat     list_fd_stat;
  char            *current_field_data;

  extra_field_info = (struct _sh_field_info *) extra_field_info_b;
  field = current_field (Sh_scr_ptr->form);
  cdkscreen = initCDKScreen (Sh_scr_ptr->current_win);

  if ((list_fd = sh209 (field)) > 0)
    {
      /* The first two bytes is the read length */
      if (read (list_fd, list_disp, 2) != 2)
        eng_error_out ("Bad field list file read");
      else
        {
          *(list_disp+2) = '\0';
          list_rd_len = atoi (list_disp);
          list_buffer = (char *) eng_malloc (list_rd_len + 1, "item_flist: list_buffer");
          *(list_buffer+list_rd_len) = '\0';
          box_width = (list_rd_len - field->maxgrow) + 3;

          fstat (list_fd, &list_fd_stat);
          item_count = list_fd_stat.st_size / list_rd_len;
          list_array = (char **) calloc(item_count, sizeof(char *));
          value_array = (char **) calloc(item_count, sizeof(char *));
          current_field_data = (char *) eng_malloc (field->maxgrow,"item_flist: current_field_data");
          memcpy (current_field_data, field_buffer (field, 0), field->maxgrow);

          current_item = 0;
          while (read (list_fd, list_buffer, list_rd_len) == list_rd_len)
            {
              *(list_buffer+field->maxgrow) = '\0';
              value_array[current_item] = eng_malloc(field->maxgrow+1,"item_flist: value_array");
              strcpy (value_array[current_item], list_buffer);
              if (memcmp(value_array[current_item], current_field_data, field->maxgrow) == 0)
                default_item = current_item;

              list_array[current_item] = eng_malloc(strlen(list_buffer+field->maxgrow+1) + 1,"item_flist: list_array");
              strcpy (list_array[current_item],list_buffer+field->maxgrow+1);
              current_item++;
            }
          free (list_buffer);
          free (current_field_data);

          if (isdigit (*extra_field_info->bval))
          {
            if ((box_height = cnvbin (extra_field_info->bval, 2)) < 3)
              box_height += (3 - box_height);
          }
          else
            box_height = 3;

            selection = newCDKSelection (cdkscreen, Sh_scr_ptr->current_win->_begx+field->fcol, Sh_scr_ptr->current_win->_begy+field->frow+1, box_height < 4 ? NONE : RIGHT, box_height, box_width, NULL, list_array, item_count, choices, 2, A_REVERSE, TRUE, FALSE);

          if (selection != 0)
            {
              /* Set the highlight field */
              setCDKSelectionCurrent (selection, default_item);

              /* Activate the widget. */
              eng_activateCDKSelection (selection);

              choice = -1;
              for (i=0; i < item_count; i++)
              {
                if (selection->selections[i] == 1)
                {
                  choice = i;
                  break;
                }
              }

              /* Check how they exited from the widget. */
              if (selection->exitType == vNORMAL && choice >= 0)
                {
                  if (set_field_buffer (field, 0, value_array[choice]) != E_OK)    /* Give the field data */
                    return (sh998("item_flist: set_field_buffer",ENG_SH_NC));
                  return_val = 1;
                }
              else
                {
                  /* This hack rewrites the underlining screen */
                  set_field_buffer (field, 0, make_string (field_buffer (field, 0),field->maxgrow,0,0,0,0));
                }
              destroyCDKSelection (selection);
            }

          destroyCDKScreen (cdkscreen);
          for (i=0;i < item_count;i++)
            {
              free (list_array[i]);
              free (value_array[i]);
            }
          free (list_array);
          free (value_array);
        }
      close (list_fd);
    }
  return (return_val);
}

int
item_list ()
{
  int             from = 0, to = 0;
  int             return_val = 0 ;
  FIELD          *field;
  CDKSCREEN      *cdkscreen = 0;
  CDKITEMLIST    *item_list = 0;
  char          **list_array = 0;
  char          **value_array = 0;
  int             item_count = 0, current_item = 0, i, choice, default_item = 0;
  char           *disp;
  char           *token;
  char           *current_field_data;
  char            scratch_list[sizeof (Sh_temp03->list)];
  char            cords[5];

  field = current_field (Sh_scr_ptr->form);
  cdkscreen = initCDKScreen (Sh_scr_ptr->current_win);

  current_field_data = field_buffer (field, 0);

  sprintf (cords, "%02d%02d", field->frow + 1 + Sh_scr_ptr->line_adj, field->fcol + 1);
  memcpy (Sh_fh_3.fhkv_row_col, cords, 4);
  Sh_fh_3.hd.current_id.ul = ENG_FH_4STARS;
  while (1)
  {
    tifh (Sh_fh_3.hd.id);
    if (Sh_fh_3.hd.status.ui != ENG_FH_OK)
    {
        /* inform of error */
      Sh_sh->status.ui = Sh_fh_3.hd.status.ui;
      return (-1);
    }

    if (Sh_fh_3.hd.current_id.ul == ENG_FH_4STARS)
      break;

    memcpy (scratch_list, Sh_temp03->list, sizeof (Sh_temp03->list) - 1);
    for (i=sizeof (Sh_temp03->list)-1; i > 0 && *(scratch_list+(i-1)) == ' '; i--);
    *(scratch_list+i) = '\0';
    disp = NULL;

    /* Count the elements */
    if (memcmp (scratch_list,"!#NUM#!", 7) == 0)
    {
      if ((token = strtok_r(scratch_list+7, "-", &disp)) != NULL)
      {
        from = atoi (token);
        token = strtok_r(NULL, "-", &disp);
        to = atoi (token);
        item_count += to - from;
      }
      else
        return (-1);
        /* Put the list back */
      memcpy (scratch_list, Sh_temp03->list, sizeof (Sh_temp03->list));
      disp = NULL;
    }
    else
    {
      for (i=0;i < sizeof (scratch_list);i++)
      {
        if (*(scratch_list+i) == ',')
          item_count++;
      }
      item_count++;
    }

    list_array = (char **) realloc(list_array, item_count * sizeof(char *));
    value_array = (char **) realloc(value_array, item_count * sizeof(char *));

    if (memcmp (scratch_list,"!#NUM#!", 7) == 0)
    {
      if ((token = strtok_r(scratch_list+7, "-", &disp)) != NULL)
      {
        from = atoi (token);
        token = strtok_r(NULL, "-", &disp);
        to = atoi (token);

        for (i=from; i <= to; i++)
        {
          value_array[current_item] = eng_malloc(field->maxgrow+1,"item_list: value_array");
          snprintf (value_array[current_item], field->maxgrow+1, "%d", i);
          if (cnvbin(value_array[current_item], field->maxgrow) == cnvbin(current_field_data, field->maxgrow))
            default_item = current_item;

          list_array[current_item] = eng_malloc(field->maxgrow+1,"item_list: list_array");
          snprintf (list_array[current_item], field->maxgrow+1, "%d", i);
          current_item++;
        }
      }
    }
    else
    {
      if ((token = strtok_r(scratch_list, ",", &disp)) != NULL)       /* intial separation */
      {
        do
        {
          value_array[current_item] = eng_malloc(field->maxgrow+1,"item_list: value_array");
          memcpy (value_array[current_item], token, field->maxgrow);
          *(value_array[current_item]+field->maxgrow) = '\0';
          if (memcmp(value_array[current_item], current_field_data, field->maxgrow) == 0)
            default_item = current_item;

          list_array[current_item] = eng_malloc(strlen(token),"item_list: list_array");
          strcpy (list_array[current_item],token+field->maxgrow+1);
          current_item++;
        }
        while((token = strtok_r(NULL, ",", &disp)) != NULL);
      }
    }
  }

  item_list = newCDKItemlist (cdkscreen,Sh_scr_ptr->current_win->_begx+field->fcol, Sh_scr_ptr->current_win->_begy+field->frow+1, NULL, NULL, list_array, item_count, default_item, TRUE,FALSE);

  if (item_list != 0)
    {
      /* Activate the widget. */
      choice = eng_activateCDKItemlist (item_list);

      /* Check how they exited from the widget. */
      if (item_list->exitType == vNORMAL && choice >= 0)
        {
          if (set_field_buffer (field, 0, value_array[choice]) != E_OK)    /* Give the field data */
            return (sh998("item_list: set_field_buffer",ENG_SH_NC));
          return_val = 1;
        }
      else
        {
          /* This hack rewrites the underlining screen */
          set_field_buffer (field, 0, make_string (field_buffer (field, 0),field->maxgrow,0,0,0,0));
        }
      destroyCDKItemlist (item_list);
    }

  destroyCDKScreen (cdkscreen);
  for (i=0;i < item_count;i++)
    {
      free (list_array[i]);
      free (value_array[i]);
    }
  return (return_val);
}

int
marquee_fields (WINDOW *win)
{
  int             i,j,half_cols,got_one;
  char           *buffer;
  char            flip_active = 0;
  FIELD          *save_field;
  FIELD          *field;
  struct _sh_field_info *user_ptr;

  eng_debug ("marquee_fields - Check for overloaded fields");

  got_one = 0;
  if (Sh_scr_ptr->form)
    {
      for (i = 0; i < Sh_scr_ptr->num_field; i++)
        {
          if ((Sh_scr_ptr->form->field[i]->cols < Sh_scr_ptr->form->field[i]->maxgrow) && (Sh_scr_ptr->form->field[i] != current_field (Sh_scr_ptr->form)) && SH_PUBLIC (Sh_scr_ptr->form->field[i]))
            {
              field = Sh_scr_ptr->form->field[i];
              buffer = field_buffer(field,0);

              /* See if there is any off screen data */
              for (j=field->cols; j < field->maxgrow; j++)
                {
                  if (*(buffer + j) != ' ')
                    {
                      break;
                    }
                }

              if (j != field->maxgrow)
                {
                  got_one = 1;
                  user_ptr = (struct _sh_field_info *) field_userptr(field);
                  if (user_ptr->cur_col >= 0)
                    {
                      save_field = current_field (Sh_scr_ptr->form);

                      /* We have to make the field selectable */
                      if (!SH_ACTIVE (field))
                      {
                        field->opts |= (O_ACTIVE);
                        flip_active = 1;
                      }

                      set_current_field (Sh_scr_ptr->form, field);
                      half_cols = field->cols / 2;

                      if (user_ptr->cur_col == 0)
                        user_ptr->cur_col = half_cols;

                      /* Scroll the field to the right */
                      for (j=user_ptr->cur_col; j >= 0; j--)
                        {
                          if (form_driver (Sh_scr_ptr->form, REQ_NEXT_CHAR) != E_OK)
                            {
                              /* If we reached the end of the field skip the next
                                 11 moves so it pauses before it goes back to the beg */
                              user_ptr->cur_col = -11;
                              break;
                            }
                        }
                      user_ptr->cur_col++;

                      /* If the field was protected re-protect it */
                      if (flip_active)
                      {
                        field->opts &= ~(O_ACTIVE);
                        flip_active = 0;
                      }

                      /* Control how far it scrolls off screen */
                      for (j=(user_ptr->cur_col - half_cols); j > 0 && j < field->maxgrow; j++)
                        {
                          if (*(buffer + j) != ' ')
                            {
                              break;
                            }
                        }
                      if (j == field->maxgrow)
                          user_ptr->cur_col = -10;

                      set_current_field (Sh_scr_ptr->form, save_field);
                      wrefresh (win);
                    }
                  else
                    {
                      user_ptr->cur_col++;
                    }
                }
            }
        }
    }
  return (got_one);
}

/*
 * This allows the user to play with the widget.
 */
int eng_activateCDKItemlist (CDKITEMLIST *itemlist)
{
  int ret = -1;
  chtype input = 0;
  boolean functionKey;

  /* Draw the widget. */
  drawCDKItemlist (itemlist, ObjOf(itemlist)->box);
  drawCDKItemlistField(itemlist, TRUE);


  for (;;)
    {
      /* Make the cursor disapear */
      raw ();
      curs_set (0);
      input = getchCDKObject (ObjOf(itemlist), &functionKey);
      curs_set (1);

      /* Inject the character into the widget. */
      if (input == KEY_UP || input == KEY_DOWN || input == KEY_ENTER || input == SPACE || input == KEY_TAB || input == KEY_ESC)
        {
          ret = injectCDKItemlist (itemlist, input);
          if (itemlist->exitType != vEARLY_EXIT)
            {
              return (ret);
            }
        }
      else
        {
          sh_fired_key = input;
          ret = -1;
          break;
        }
    }

  return (ret);
}

/*
 * This allows the user to play with the widget.
 */
int eng_activateCDKSelection (CDKSELECTION *selection)
{
  int ret = -1;
  chtype input = 0;
  boolean functionKey;

  /* Draw the selection list */
  drawCDKSelection (selection, ObjOf(selection)->box);

  for (;;)
  {
    /* Make the cursor disapear */
    raw ();
    curs_set (0);
    input = getchCDKObject (ObjOf(selection), &functionKey);
    curs_set (1);

    /* Inject the character into the widget. */
    if (input == KEY_UP || input == KEY_DOWN || input == KEY_ENTER || input == SPACE || input == KEY_TAB || input == KEY_ESC)
    {
      if (input == KEY_ENTER || input == KEY_TAB)
        injectCDKSelection (selection, SPACE);
      ret = injectCDKSelection (selection, input);
      if (selection->exitType != vEARLY_EXIT)
        return (ret);
    }
    else
    {
      sh_fired_key = input;
      ret = -1;
      break;
    }
  }

  return (ret);
}

void sh_info ()
{
  CDKSCREEN    *cdkscreen;
  CDKDIALOG    *question  = 0;
  char         *buttons[] = {"OK"};
  char         *mesg[4];
  char          buffer[60];
  int           len;

    /* Set up CDK. */
  cdkscreen = initCDKScreen (Sh_scr_ptr->current_win);

  sprintf (buffer, "ID: %2.2s%cProgram: %8.8s%cScreen: %8.8s%cField: %02d%02d", Sh_sh->id+8, '\0', Sh_sh->id, '\0', Sh_sh->screen, '\0', Sh_scr_ptr->form->current->frow, Sh_scr_ptr->form->current->fcol);

  mesg[0] = buffer;
  len = strlen (buffer) + 1;
  mesg[1] = buffer + len;
  len += strlen (buffer + len) + 1;
  mesg[2] = buffer + len;
  len += strlen (buffer + len) + 1;
  mesg[3] = buffer + len;

    /* Create the dialog box. */
  question = newCDKDialog (cdkscreen, CENTER, CENTER, mesg, 4, buttons, 1, sh603 (SH_FORE_BLACK, SH_BACK_WHITE, SH_UNPRO_TYPE), TRUE, TRUE,              FALSE);

    /* Is the label null? */
  if (question == 0)
  {
      /* Clean up the memory. */
    destroyCDKScreen (cdkscreen);
    return;
  }

    /* Turn the cursor off */
  curs_set (0);

    /* Activate the dialog box. */
  activateCDKDialog (question, 0);

    /* Cursor back on */
  curs_set (1);

    /* Clean up. */
  destroyCDKDialog (question);
  destroyCDKScreen (cdkscreen);
  sh111 ();
}

void
sh101t ()
{}

