/*
*  C Implementation: sh603
*
* Description: Return the real curses color code with attrib
*
*
* Author: Ricardo Cantu <ricky@smartcsc.com>, (C) 2006
*
* Copyright: See COPYING file that comes with this distribution
*
*/

chtype
sh603 (unsigned char color, unsigned char background, unsigned char type)
{
  int             pair;
  int             attrib;
  short int       fore, back;

  eng_debug ("sh603: Get color: %c", color);

  if (color == DEFAULT_UPRO)
    color = Sh_def_unprotected;
  else if (color == DEFAULT_PRO)
    {
      if (type == SH_PRO_TYPE)
        color = Sh_def_protected;
      else
        color = Sh_def_unprotected;
    }

    for (pair=1; pair < 15; pair++)
    {
      if (Sh_color_pair[pair].fore == color && Sh_color_pair[pair].back == background)
      {
        eng_debug ("sh603: Found color cached");
        break;
      }

      if (!Sh_color_pair[pair].fore)
        {
          eng_debug ("sh603: Create new pair: %c:%c", color, background);
          Sh_color_pair[pair].fore = color;
          Sh_color_pair[pair].back = background;
          fore = fetch_color (color);
          Sh_color_pair[pair].html_fore = Sh_html_color;
          back = fetch_color (background);
          Sh_color_pair[pair].html_back = Sh_html_color;
          if (init_pair (pair, fore, back) == ERR)
            return (eng_error_out("sh603: Error in init_pair"));
          break;
        }
    }
    attrib = fetch_attrib (color);
    Sh_color_pair[pair].attrib = attrib;

  return (COLOR_PAIR (pair) | attrib);
}

short int
fetch_color (unsigned char color)
{
  short int       cur_color;

  switch (color)
    {
    case '@':
    case '\x60':
      {
        cur_color = COLOR_BLACK;
        Sh_html_color = Sh_html_colors[SH_HTML_BLACK];
        break;
      }
    case 'A':
    case 'a':
      {
        cur_color = COLOR_BLUE;
        Sh_html_color = Sh_html_colors[SH_HTML_BLUE];
        break;
      }
    case 'B':
    case 'b':
      {
        cur_color = COLOR_YELLOW;
        Sh_html_color = Sh_html_colors[SH_HTML_YELLOW];
        break;
      }
    case 'C':
    case 'c':
      {
        cur_color = COLOR_CYAN;
        Sh_html_color = Sh_html_colors[SH_HTML_CYAN];
        break;
      }
    case 'D':
    case 'd':
      {
        cur_color = COLOR_RED;
        Sh_html_color = Sh_html_colors[SH_HTML_RED];
        break;
      }
    case 'E':
    case 'e':
      {
        cur_color = COLOR_MAGENTA;
        Sh_html_color = Sh_html_colors[SH_HTML_MAGENTA];
        break;
      }
    case 'F':                                             /* Brown */
    case 'f':
      {
        cur_color = COLOR_YELLOW;
        Sh_html_color = Sh_html_colors[SH_HTML_BROWN];
        break;
      }
    case 'G':                                             /* Beige */
    case 'g':
      {
        cur_color = COLOR_WHITE;
        Sh_html_color = Sh_html_colors[SH_HTML_BEIGE];
        break;
      }
    case 'H':
    case 'h':                                             /* Grey */
      {
        cur_color = COLOR_WHITE;
        Sh_html_color = Sh_html_colors[SH_HTML_GRAY];
        break;
      }
    case 'I':
    case 'i':                                             /* Light blue */
      {
        cur_color = COLOR_BLUE;
        Sh_html_color = Sh_html_colors[SH_HTML_LIGHT_BLUE];
        break;
      }
    case 'J':
    case 'j':                                             /* Light green */
      {
        cur_color = COLOR_GREEN;
        Sh_html_color = Sh_html_colors[SH_HTML_LIGHT_GREEN];
        break;
      }
    case 'K':
    case 'k':                                             /* Light cyan */
      {
        cur_color = COLOR_CYAN;
        Sh_html_color = Sh_html_colors[SH_HTML_LIGHT_CYAN];
        break;
      }
    case 'L':
    case 'l':                                             /* Light Red */
      {
        cur_color = COLOR_RED;
        Sh_html_color = Sh_html_colors[SH_HTML_LIGHT_RED];
        break;
      }
    case 'M':
    case 'm':                                             /* Light Magenta */
      {
        cur_color = COLOR_MAGENTA;
        Sh_html_color = Sh_html_colors[SH_HTML_LIGHT_MAGENTA];
        break;
      }
    case 'N':
    case 'n':                                             /* Yellow */
      {
        cur_color = COLOR_YELLOW;
        Sh_html_color = Sh_html_colors[SH_HTML_YELLOW];
        break;
      }
    case 'O':
    case 'o':
      {
        cur_color = COLOR_WHITE;
        Sh_html_color = Sh_html_colors[SH_HTML_WHITE];
        break;
      }
    case 'Q':
    case 'q':
    {
      cur_color = COLOR_BLACK;                        /* Does not exist */
      Sh_html_color = Sh_html_colors[SH_HTML_WHITESMOKE];
      break;
    }
    default:
      {
        cur_color = COLOR_WHITE;
        Sh_html_color = Sh_html_colors[SH_HTML_BEIGE];
        break;
      }
    }

  return (cur_color);
}

int
fetch_attrib (unsigned char color)
{

  int             cur_attrib;

  switch (color)
    {
    case '@':                                             /* Black */
    case '\x60':
    case 'G':                                             /* White */
    case 'g':
      cur_attrib = A_NORMAL;
      break;

    case 'B':                                             /* Blink Yellow */
    case 'b':
      cur_attrib = A_BLINK;
      break;

    case 'F':                                             /* Brown */
    case 'f':
    case 'H':                                             /* Grey */
    case 'h':
    case 'I':                                             /* Light blue */
    case 'i':
    case 'J':                                             /* Light green */
    case 'j':
    case 'K':                                             /* Light cyan */
    case 'k':
    case 'L':                                             /* Light Red */
    case 'l':
    case 'M':                                             /* Light Magenta */
    case 'm':
      cur_attrib = A_DIM;
      break;

    case 'A':                                             /* Blue */
    case 'a':
    case 'C':                                             /* Cyan */
    case 'c':
    case 'D':                                             /* Red */
    case 'd':
    case 'E':                                             /* Magenta */
    case 'e':
    case 'N':                                             /* Yellow */
    case 'n':
    case 'O':                                             /* High White */
    case 'o':
      cur_attrib = A_BOLD;
      break;

    default:
      cur_attrib = A_NORMAL;
      break;
    }

  return (cur_attrib);
}
