/*
*  C Implementation: sh_curses
*
* Description: Inhouse engine to curses/fcgi translations
*
*
* Author: Ricardo Cantu <ricky@smartcsc.com>, (C) 2006
*
* Copyright: See COPYING file that comes with this distribution
*
*/

int eng_wrefresh (WINDOW *win)
{
  int     i;
  int     zindex;

  if (Sh_flags.screen_mode == SH_TEXT)
    {
      if (Sh_scr_ptr->form)               /* Set the real cursor position within the field on the form */
        {
          if (Sh_scr_ptr->form->current->frow == Sh_scr_ptr->cur_row && Sh_scr_ptr->cur_col <= (Sh_scr_ptr->form->current->fcol+Sh_scr_ptr->form->current->maxgrow))
            {
              for (i=Sh_scr_ptr->form->current->fcol; i < Sh_scr_ptr->cur_col; i++)
                form_driver (Sh_scr_ptr->form, REQ_NEXT_CHAR);
            }
        }
      wrefresh (win);

      /* Make sure form cursor is in the right place */
      pos_form_cursor(Sh_scr_ptr->form);
      return (wrefresh (win));
    }
  else
    {
      if (Sh_scr_ptr->is_popup)
        {
          /* popup screen */
          Sh_fcgi_header.display_type = '1';
          /* Window size */
          eng_style_dimensions (win->_maxy + 1, win->_maxx + 1);
          zindex = 1000;
        }
      else
        {
          /* reg screen */
          Sh_fcgi_header.display_type = '0';
          /* Window size */
          eng_style_dimensions (win->_maxy + 1, win->_maxx + 1);
          zindex = 1;
        }

        /* Put Header out */
      eng_html_header ();

      /* Color and attributes of the window */
      eng_style_color_win (win);

      /* Where window starts */
      eng_style_position (win->_begy, win->_begx, 0, 0);
      if (Sh_fcgi_header.display_type == '1')
        bassign (Sh_div_dragable, &Sh_div_drag);
      else
        btrunc (Sh_div_dragable, 0);
      bformata (Sh_html_buffer, "\n<div id=\"current_scr\" %s style=\"z-index: %d; %s %s %s font-family: %s; font-size: %dpt; margin: 0; padding: 0;\">", Sh_div_dragable->data, zindex, Sh_style_position->data, Sh_style_dimensions->data, Sh_style_color->data, Sh_html_fonts[SH_HTML_FONT_NAME], SH_HTML_FONT_SIZE);

      /* Put out protected data */
      bconcat (Sh_html_buffer, Sh_html_pro_buffer);

      /* Put out unprotected data */
      bconcat (Sh_html_buffer, &Sh_html_form_beg);
      bconcat (Sh_html_buffer, Sh_html_unpro_buffer);

      bformata (Sh_html_buffer, "\n<input id=\"cursor_pos\" type=\"hidden\" name=\"cursor_pos\" maxlength=\"4\" size=\"4\" value=\"%02d%02d\">", win->_cury + 1, win->_curx + 1);
      bformata (Sh_html_buffer, "\n<input id=\"focus_name\" type=\"hidden\" name=\"focus_name\" maxlength=\"4\" size=\"4\" value=\"%s\">", Sh_focus_name->data);
      bcatcstr (Sh_html_buffer, "\n<input id=\"keys\" type=\"hidden\" name=\"keys\" maxlength=\"2\" size=\"2\" value=\"\">");
      bcatcstr (Sh_html_buffer,"\n</form>\n</div>\n");

      eng_debug ("eng_wrefresh: data:");
      eng_debug_low ((char *) Sh_html_buffer->data, Sh_html_buffer->slen);

      do
      {
        FCGX_PutStr((char *) Sh_html_buffer->data, Sh_html_buffer->slen, Sh_fcgi_io.out);
        if (FCGX_Accept_r (&Sh_fcgi_io) < 0)
          return (eng_error_out (Eng_error_mesg[14], "eng_wrefresh: FCGX_Accept_r", strerror (errno)));

        /* Wait for a good response */
      } while (sh121 () < 0);


      /* Indicate that we wrote a page */
      Sh_wrote_html = ON;
    }
  return (0);
}


int eng_waddstr (WINDOW *win, char *data, char font, char style, char *size)
{
  short int   p_font;

  if (Sh_flags.screen_mode == SH_TEXT)
    {
      return (waddstr (win, data));
    }
  else
    {
      /* Where the data goes */
      eng_style_position (win->_cury, win->_curx, 0, 0);

      btrunc (Sh_style_decoration, 0);
      if (isdigit (*size))
        bformata (Sh_style_decoration,"font-size: %spt;",size);

      p_font = eng_alnum_to_int (font);
      if (p_font >= 0 && p_font <= 3)
        bformata (Sh_style_decoration,"font-family: %s;",Sh_html_fonts[p_font]);

      /* Color and attributes of the window */
      /* For some reason this causes windows to make line drawing chars not connect correctly */
      //eng_style_color_win (win);

      /* Actual data */
      bformata (Sh_html_pro_buffer, "\n<span style=\"%s %s\">%s</span>", Sh_style_position->data, Sh_style_decoration->data, data);
    }
  return (0);
}



int eng_post_form (FORM *form)
{
  void          build_select(FIELD *);

  short int     i,tabindex,bcap_len,bcap_adj,trailing_spaces;
  short int     y_cord,x_cord;
  short int     u_font;
  char          *buffer;
  char          *token;
  struct        _sh_field_info *extra_field_info;             /* utyp,tcap,wcap */
  FIELD         *field;

  eng_debug ("eng_post_form: Processing form buffer");

  if (Sh_flags.screen_mode == SH_TEXT)
    {
      return (post_form (form));
    }
  else
    {
      /* Do the unprotected data now */
      /* Setup defaults for all fields */
      bassignformat (Sh_html_unpro_buffer, "\n<style type=\"text/css\"><!-- input {font-family: %s; font-size: %dpt; border-style: none; border: 0; margin: 0; padding: 0;} --></style>", Sh_html_fonts[SH_HTML_FONT_NAME], SH_HTML_FONT_SIZE);

      tabindex = 0;
      for (i = 0; i < form->maxfield; i++)
        {
          field = form->field[i];
          extra_field_info = (struct _sh_field_info *) field_userptr(field);
          u_font = eng_alnum_to_int (extra_field_info->ufnt);

          if (extra_field_info->utyp == SH_TYPE_CHECK)
            {
              eng_style_position (field->frow, field->fcol, SH_HTML_ROW_PIXEL / 4, 0);
              eng_style_color_form (field);
            }
          else if (extra_field_info->utyp == SH_TYPE_BUTTON)
            {
              bcap_len = strlen(extra_field_info->bcap);
              if (*extra_field_info->bval != ' ' && *extra_field_info->bval != '\0')
                {
                  bcap_adj = 1;
                }
              else
                {
                  bcap_adj = 0;
                }
              eng_style_position (field->frow, field->fcol - ((bcap_len+bcap_adj) - 0), 0, 0);
              eng_style_button (bcap_len);
            }
          else
            {
              eng_style_position (field->frow, field->fcol, 0, 0);
              eng_style_color_form (field);
            }

            /* Adjust to start from 1 */
            y_cord = field->frow + 1 + Sh_scr_ptr->line_adj;
            x_cord = field->fcol + 1;

          /* See if this field has focus */
          if (field == form->current)
            bassignformat (Sh_focus_name, "%02d%02d", y_cord, x_cord);

          /* See if we have a tooltip */
          if (extra_field_info->wcap)
            bassignformat (Sh_mouse_over_out, "onmouseover=\"sh_init_tooltip(event,this,'%s')\" onmouseout=\"sh_stop_tooltip()\" ", extra_field_info->wcap);
          else
            btrunc (Sh_mouse_over_out, 0);

          bassignformat (Sh_field_name_id, "name=\"%02d%02d\" id=\"%02d%02d\"", y_cord, x_cord, y_cord, x_cord);
          bassignformat (Sh_input_focus, "onfocus=\"eng_handle_focus(event,this,'focus')\" onblur=\"eng_handle_focus(event,this,'blur')\" autocomplete=\"off\"  tabindex=\"%d\"",++tabindex);

          bassignformat (Sh_field_html_size, "maxlength=\"%d\" size=\"%d\"",field->maxgrow, field->cols);

          /* Check font and style settings */
          bdelete (Sh_style_font, 0, Sh_style_font->slen);
          if (extra_field_info->usty == SH_HTML_STYLE_BOX)
            bconcat (Sh_style_font, &Sh_style_border_box);

          if (u_font >= 0 && u_font <= 3)
            bformata (Sh_style_font, "font-family: %s;", Sh_html_fonts[u_font]);

          if (isdigit (*extra_field_info->ufsz))
            bformata (Sh_style_font,"font-size: %spt;", extra_field_info->ufsz);

          /* Make the actual fields */
          if (!SH_ACTIVE (field))
            {
              /* Protected Text box */
              if (SH_PUBLIC (field))
                {
                  /* Displayed */
                  if (extra_field_info->utyp == SH_TYPE_LINK)
                    {
                      token = make_string (field_buffer (field, 0), field->cols,1,0,0,0);

                      bformata (Sh_html_unpro_buffer, "\n<a href=\"#\" onclick=\"sh_button_click('%s','%s','%s')\"  style=\"%s color: #DC143C;\">%s</a>",token, extra_field_info->bkey, extra_field_info->bval, Sh_style_position->data, token);
                      tabindex--;
                    }
                  else if (extra_field_info->utyp == SH_TYPE_BUTTON)
                    {
                      bformata (Sh_html_unpro_buffer, "\n<input type=\"button\" name=\"B%02d%02d\" id=\"B%02d%02d\" onmousedown=\"sh_change_background(this,'#FF0000')\" onclick=\"sh_button_click('%s','%s','%02d%02d')\" %s %s style=\"%s %s\" value=\"%s\">",y_cord, x_cord, y_cord, x_cord, extra_field_info->bval, extra_field_info->bkey, y_cord, x_cord, Sh_mouse_over_out->data, Sh_input_focus->data, Sh_style_position->data, Sh_style_button->data, extra_field_info->bcap);
                    }
                  else
                    {
                      if (field->cols == field->maxgrow)
                        {
                          bformata (Sh_html_unpro_buffer, "\n<span style=\"%s %s\">%s</span>",Sh_style_position->data, Sh_style_color->data ,make_string (field_buffer (field, 0),field->cols,0,0,3,0));
                          tabindex--;
                        }
                      else
                        {
                          bformata (Sh_html_unpro_buffer, "\n<input disabled type=\"text\" %s %s style=\"%s %s\" value=\"%s\">",Sh_field_html_size->data, Sh_input_focus->data, Sh_style_position->data, Sh_style_color->data, make_string (field_buffer (field, 0), field->maxgrow,1,0,3,0));
                        }
                    }
                }
                else
                {
                  /* Protected and non-display, don't even send over */
                  tabindex--;
                }
            }
          else
            {
              if (extra_field_info->utyp == SH_TYPE_CHECK)
                {
                  buffer = field_buffer (field, 0);
                  if (*buffer == ' ' || *buffer == 'N')
                    bformata (Sh_html_unpro_buffer, "\n<input type=\"checkbox\" %s %s %s %s style=\"%s %s\">", Sh_mouse_over_out->data, Sh_field_name_id->data, Sh_input_focus->data, Sh_field_html_size->data, Sh_style_position->data, Sh_style_color->data);
                  else
                    bformata (Sh_html_unpro_buffer, "\n<input type=\"checkbox\" %s %s %s %s style=\"%s %s\" checked>", Sh_mouse_over_out->data, Sh_field_name_id->data, Sh_input_focus->data, Sh_field_html_size->data, Sh_style_position->data, Sh_style_color->data);
                }
              else if (extra_field_info->utyp == SH_TYPE_BUTTON)
                {
                  bformata (Sh_html_unpro_buffer, "\n<input type=\"button\" name=\"B%02d%02d\" id=\"B%02d%02d\" onmousedown=\"sh_change_background(this,'#FF0000')\" onclick=\"sh_button_click('%s','%s','%02d%02d')\" %s %s style=\"%s %s\" value=\"%s\">",y_cord, x_cord, y_cord, x_cord, extra_field_info->bval, extra_field_info->bkey, y_cord, x_cord, Sh_mouse_over_out->data, Sh_input_focus->data, Sh_style_position->data, Sh_style_button->data, extra_field_info->bcap);

                  bformata (Sh_html_unpro_buffer, "\n<input type=\"hidden\" tabindex=\"%d\" %s %s style=\"%s\" value=\"%s\">", ++tabindex, Sh_field_name_id->data, Sh_field_html_size->data, Sh_style_position->data, make_string (field_buffer (field, 0),field->cols,1,0,0,0));
                }
              else if (extra_field_info->utyp == SH_TYPE_LIST)
                {
                  /* We need to send the size over in the name */
                  bassignformat (Sh_field_name_id, "name=\"%02d%02d%02d\" id=\"%02d%02d\"", y_cord, x_cord, field->maxgrow, y_cord, x_cord);

                  bformata (Sh_html_unpro_buffer, "\n<select %s %s onchange=\"sh_list_select('%s')\" style=\"%s\">",Sh_field_name_id->data, Sh_input_focus->data, extra_field_info->bkey, Sh_style_position->data);

                  build_select (field);

                  bcatcstr (Sh_html_unpro_buffer, "\n</select>");
                }
              else if (extra_field_info->utyp == SH_TYPE_FLST)
                {
                  char  list_disp[3];
                  char  *list_buffer, *current_field_data;
                  int   list_fd,list_rd_len;
                  int   list_dsize;

                  /* We need to send the size over in the name */
                  bassignformat (Sh_field_name_id, "name=\"%02d%02d%02d\" id=\"%02d%02d\"", y_cord, x_cord, field->maxgrow, y_cord, x_cord);

                  if (isdigit (*extra_field_info->bval) && isdigit(*(extra_field_info->bval+1)))
                    list_dsize = cnvbin (extra_field_info->bval, 2);
                  else
                    list_dsize = 1;

                  bformata (Sh_html_unpro_buffer, "\n<select %s %s size=\"%d\" style=\"%s\">",Sh_field_name_id->data,Sh_input_focus->data, list_dsize, Sh_style_position->data);

                  current_field_data = field_buffer (field, 0);
                  if ((list_fd = sh209(field)) > 0)
                    {
                      if (read (list_fd, list_disp, 2) != 2)
                        eng_error_out ("eng_post_form: Bad field list file read");
                      else
                        {
                          *(list_disp+2) = '\0';
                          list_rd_len = atoi (list_disp);
                          list_buffer = (char *) eng_malloc (list_rd_len + 1, "sh_curses: list_buffer");
                          *(list_buffer+list_rd_len) = '\0';
                          while (read (list_fd, list_buffer, list_rd_len) == list_rd_len)
                            {
                              *(list_buffer+field->cols) = '\0';
                              if (memcmp (current_field_data, list_buffer, field->cols) == 0)
                                bformata (Sh_html_unpro_buffer, "\n<option selected value=\"%s\">%s</option>",list_buffer,list_buffer+field->cols+1);
                              else
                                bformata (Sh_html_unpro_buffer, "\n<option value=\"%s\">%s</option>",list_buffer,list_buffer+field->cols+1);
                            }
                          free (list_buffer);
                        }
                      close (list_fd);
                    }
                  bcatcstr (Sh_html_unpro_buffer, "\n</select>");
                }
              else
                {
                  if (!SH_PUBLIC (field))
                    {
                      /* Like passwords */
                      bformata (Sh_html_unpro_buffer, "\n<input type=\"password\" %s %s %s %s style=\"%s %s\" value=\"%s\">", Sh_mouse_over_out->data, Sh_field_name_id->data, Sh_input_focus->data, Sh_field_html_size->data, Sh_style_position->data, Sh_style_color->data, make_string (field_buffer (field, 0), field->maxgrow,1,0,0,0));
                    }
                  else
                    {
                      /* Reglular input text box */
                      if (eng_char_of (field->pad) != ' ' && field->pad)
                        bconcat (Sh_style_decoration, &Sh_style_unders);
                      else
                        btrunc (Sh_style_decoration, 0);

                      if (field->maxgrow != field->cols)
                        trailing_spaces = 1;
                      else
                        trailing_spaces = 0;

                      bformata (Sh_html_unpro_buffer, "\n<input type=\"text\" onclick=\"eng_handle_click(event,this)\" %s %s %s %s style=\"%s %s %s %s\" value=\"%s\">", Sh_mouse_over_out->data, Sh_field_name_id->data, Sh_input_focus->data, Sh_field_html_size->data, Sh_style_position->data, Sh_style_color->data, Sh_style_decoration->data, Sh_style_font->data, make_string (field_buffer (field, 0), field->maxgrow,trailing_spaces,0,3,0));
                    }
                }
            }
        }
    }
  return (0);
}

void build_select (FIELD *field)
{
  int     from = 0, to = 0;
  int     i;
  char   *disp, *token, *current_field_data;
  char    scratch_list[sizeof (Sh_temp03->list)];
  char    cords[5];

  current_field_data = field_buffer (field, 0);

  sprintf (cords, "%02d%02d", field->frow + 1 + Sh_scr_ptr->line_adj, field->fcol + 1);
  memcpy (Sh_fh_3.fhkv_row_col, cords, 4);
  Sh_fh_3.hd.current_id.ul = ENG_FH_4STARS;
  while (1)
  {
    tifh (Sh_fh_3.hd.id);
    if (Sh_fh_3.hd.status.ui != ENG_FH_OK)
    {
        /* inform of error */
      Sh_sh->status.ui = Sh_fh_3.hd.status.ui;
      return;
    }

    if (Sh_fh_3.hd.current_id.ul == ENG_FH_4STARS)
      break;

    memcpy (scratch_list, Sh_temp03->list, sizeof (Sh_temp03->list));
    for (i=sizeof (Sh_temp03->list)-1; i > 0 && *(scratch_list+(i-1)) == ' '; i--);
    *(scratch_list+i) = '\0';

    disp = NULL;
    if (memcmp (scratch_list,"!#NUM#!", 7) == 0)
    {
      if ((token = strtok_r(scratch_list+7, "-", &disp)) != NULL)
      {
        from = atoi (token);
        token = strtok_r(NULL, "-", &disp);
        to = atoi (token);

        for (i=from; i <= to; i++)
        {
          if (cnvbin (current_field_data, field->maxgrow) == i)
            bformata (Sh_html_unpro_buffer, "\n<option selected value=\"%-*.*d\">%d</option>", field->maxgrow, field->maxgrow, i, i);
          else
            bformata (Sh_html_unpro_buffer, "\n<option value=\"%-*.*d\">%d</option>", field->maxgrow, field->maxgrow, i, i);
        }
      }
    }
    else
    {
        /* intial separation */
      disp = NULL;
      if ((token = strtok_r(scratch_list, ",", &disp)) != NULL)
      {
        do
        {
          *(token+field->cols) = '\0';
          if (memcmp (current_field_data, token, field->cols) == 0)
            bformata (Sh_html_unpro_buffer, "\n<option selected value=\"%s\">%s</option>",token,token+field->cols+1);
          else
            bformata (Sh_html_unpro_buffer, "\n<option value=\"%s\">%s</option>",token,token+field->cols+1);
          *(token+field->cols) = '=';
        }
        while((token = strtok_r(NULL, ",", &disp)) != NULL);
      }
    }
  }
}

