/*
*  C Implementation: shinit
*
* Description: Initialize all needed to start screen handler
*
*
* Author: Ricardo Cantu <ricky@smartcsc.com>, (C) 2006
*
* Copyright: See COPYING file that comes with this distribution
*
*/

int
shinit ()
{
  void parse_query ();

  char           *env;          /* Enviroment pointer */
  char            ttylog[100];  /* Final tyy log path */
  char           *replace;      /* filter buffer */
  int             i;
  int             web_err = 0;
  int             row, colum;
  FILE           *fd;
  union rlng      msgkey;       /* message key field */
  struct _eng_config_screen config_screen;

  eng_debug ("shinit: locale: %s", setlocale (LC_ALL, "en_US.UTF-8"));

  memset (&Sh_fh, ' ', sizeof (struct _fh_parm));
  /* Identify ourselves to fh */
  memcpy (Sh_fh.hd.id, "!##!SH", 6);
  memcpy (Sh_fh.fhln, templist, strlen (templist));
  Sh_temp02 = (struct _temp02 *) Sh_fh.data;
  memcpy (Sh_fh.hd.key_name.cl, "NAME", 4);

  memcpy (Sh_fh_3.fhln, templist3, strlen (templist3));
  Sh_temp03 = (struct _temp03 *) Sh_fh_3.data;
  memcpy (Sh_fh_3.hd.key_name.cl, "LSKY", 4);
  Sh_fh_3.hd.function.ui = ENG_FH_RF;

  /* Make all the bstrings */
  Sh_focus_name = bfromcstr (" ");
  Sh_html_buffer = bfromcstr (" ");
  Sh_html_pro_buffer = bfromcstr (" ");
  Sh_html_unpro_buffer = bfromcstr (" ");
  Sh_div_dragable = bfromcstr (" ");
  Sh_style_position = bfromcstr (" ");
  Sh_style_dimensions = bfromcstr (" ");
  Sh_style_color = bfromcstr (" ");
  Sh_style_button = bfromcstr (" ");
  Sh_style_decoration = bfromcstr (" ");
  Sh_style_font = bfromcstr (" ");
  Sh_input_focus = bfromcstr (" ");
  Sh_mouse_over_out = bfromcstr (" ");
  Sh_field_name_id = bfromcstr (" ");
  Sh_field_html_size = bfromcstr (" ");
  Sh_form_type = bfromcstr (" ");

  Sh_def_background = Sh_def_sys_background = 'A';            /* Default background color */
  Sh_def_protected = Sh_def_sys_protected = 'H';              /* Default protected color */
  Sh_def_unprotected = Sh_def_sys_unprotected = 'O';          /* Default protected color */
  Sh_def_pad_char = ' ';

  /* First find out if we are web or text */
  Sh_flags.screen_mode = SH_TEXT;                          /* Start out in text mode */
  if (sh300 ())                                            /* See if fcgi */
    {
      Sh_def_background = Sh_def_sys_background = 'P';
      Sh_def_protected = Sh_def_sys_protected = 'A';
      Sh_def_unprotected = Sh_def_sys_unprotected = '@';
      Sh_flags.screen_mode = SH_WEB;                          /* Set to web mode */
      eng_debug ("shinit: Running in Web mode");

#ifdef ENG_DEBUG_FCGI
      int             listen_socket;              /* Listening Socket of port */
      char            openSocket[30];             /* Socket file to attach to for each process */
      sprintf(openSocket,"/tmp/engine_debug");
      if ((listen_socket = FCGX_OpenSocket(openSocket, 101)) < 0)
        return (eng_error_out ("shinit: Error in FCGX_OpenSocket"));
      chmod (openSocket, S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP|S_IROTH|S_IWOTH);
      if (FCGX_InitRequest (&Sh_fcgi_io, listen_socket, 0))
        return (eng_error_out ("shinit: Error in FCGX_InitRequest"));
#else
      if (FCGX_InitRequest (&Sh_fcgi_io, 0, 0))
        return (eng_error_out ("shinit: Error in FCGX_InitRequest\n"));
#endif

      FCGX_Init ();
      FCGX_Accept_r (&Sh_fcgi_io);         /* This accepts the intial request */
      parse_query ();                      /* Set all the url params as env varbs */
    }

  Sh_mast_pid = getpid ();
  if ((env = (char *) getenv ("ENG_ID")) == NULL) 
    {
      env = (char *) ttyname (0);                          /* get full device name */
      while ((replace = memchr (env, '/', strlen (env))))
        *replace = 'S';                                    /* replace slash */
      memcpy (Sh_eng_id, env + (strlen (env) - 2), 2);
    }
  else
    memcpy (Sh_eng_id, env, 2);
  *(Sh_eng_id+2) = '\0';

  if ((env = (char *) getenv ("ENG_LOG")) == NULL)
    sprintf (ttylog, "/var/log/engine/terminal_log/%s.log", Sh_eng_id);
  else
    snprintf (ttylog, sizeof (ttylog), "%s/%s.log", env, Sh_eng_id);

  /* Direct standard error to log file also */
  if ((Sh_log_fd = freopen (ttylog, "a+", stderr)) == NULL)
    return (eng_error_out ("shinit - Error creating tty log file (%s)", ttylog));

  for (i = 0; i < SH_NUM_SCREEN; i++)
    {
      Sh_scr_data[i].cur_window = 0;

      /* Create core for window struct */
      sh504 (i, 0);
    }

  /* init color array */
  memset (Sh_color_pair, '\0', sizeof (Sh_color_pair));

  memset (Sh_mesg1, ' ', SH_TMESG_LEN);
  memset (Sh_mesg2, ' ', SH_TMESG_LEN);

  msgkey.cl[0] = msgkey.cl[1] = 'T';                 /* set up key */
  msgkey.cl[2] = Sh_eng_id[0];
  msgkey.cl[3] = Sh_eng_id[1];

  if ((Sh_mesg_id = msgget (msgkey.nl, 0) == 0))
    msgctl (Sh_mesg_id, IPC_RMID, 0);                        /* kill msg queue to reinit */

  if ((Sh_mesg_id = msgget (msgkey.nl, IPC_CREAT | 0666)) < 0)
    return (eng_error_out (Eng_error_mesg[21], errno));

  sigemptyset (&Sh_sa.sa_mask);                           /* Use signaling for detached process */
  Sh_sa.sa_flags = SA_RESTART;
  Sh_sa.sa_handler = (void *) sh107c;
  sigaction (SIGUSR1, &Sh_sa, NULL);

  fd = open_config ();                                     /* Open the config file */

  if (get_parm_screen (fd, (struct _eng_config_screen *) &config_screen))       /* Get the screen defaults */
    {
      if (Sh_flags.screen_mode == SH_WEB)                                    /* Make sure we hand shake properly and exchange some info */
      { 
        if (config_screen.web_background)
          Sh_def_sys_background = config_screen.web_background;
        if (config_screen.web_pro)
          Sh_def_sys_protected = config_screen.web_pro;
        if (config_screen.web_unpro)
          Sh_def_sys_unprotected = config_screen.web_unpro;
      }
      else
      {
        if (config_screen.background)
          Sh_def_sys_background = config_screen.background;
        if (config_screen.pro)
          Sh_def_sys_protected = config_screen.pro;
        if (config_screen.unpro)
          Sh_def_sys_unprotected = config_screen.unpro;
      }

      if (config_screen.pad_char)
        Sh_def_pad_char = config_screen.pad_char;
      if (config_screen.pad_reverse)
        Sh_flags.pad_reverse = eng_alnum_to_int (config_screen.pad_reverse);
      if (config_screen.filter_pad)
        Sh_flags.filter_pad = eng_alnum_to_int (config_screen.filter_pad);
      if (config_screen.pad_top_line)
        Sh_flags.pad_top_line = eng_alnum_to_int (config_screen.pad_top_line);
      if (config_screen.hide_top_line)
        Sh_flags.hide_top_line = eng_alnum_to_int (config_screen.hide_top_line);
      if (config_screen.first_key_blank)
        Sh_flags.first_key_blank = eng_alnum_to_int (config_screen.first_key_blank);
    }

  fseek (fd, 0, SEEK_SET);        /* Reset back to beginging of file */

  Sh_beg_config_keys = Sh_config_keys = (struct _eng_config_keys_list *) eng_malloc (sizeof (struct _eng_config_keys_list), "shinit: Sh_config_keys");
  Sh_config_keys->next_eng_config_keys_list = NULL;
  get_parm_keys (fd, (struct _eng_config_keys_list *) Sh_config_keys);       /* Get the key mappings */

  fclose (fd);

  /* Make sure we hand shake properly and exchange some info */
if (Sh_flags.screen_mode == SH_WEB)
  {
    while (sh301 () < 0)
    {
      sleep (1);
      if (web_err++ > 10)
       return(-1);
    }
  }

  /* Enviro vars override config file */
  if ((env = (char *) getenv ("ENG_BACK")) != NULL)        /* Background color */
    Sh_def_sys_background = (char) *env;
  if ((env = (char *) getenv ("ENG_PRO")) != NULL)         /* Protected color */
    Sh_def_sys_protected = (char) *env;
  if ((env = (char *) getenv ("ENG_UNPRO")) != NULL)       /* Unprotected color */
    Sh_def_sys_unprotected = (char) *env;

  Sh_def_background = Sh_def_sys_background;              /* Reset these again */
  Sh_def_protected = Sh_def_sys_protected;
  Sh_def_unprotected = Sh_def_sys_unprotected;

  if (Sh_flags.detached != ON)
    {
      if (Sh_flags.screen_mode == SH_WEB)
        {
          SCREEN *screen;
          FILE   *fake;

          eng_debug ("shinit: Init web info");

          /* This allows us to use the cureses routines for storage */
          fake = fopen ("/dev/null", "a");
          if ((screen = newterm("xterm", fake, fake)) == NULL)
            return (eng_error_out("shinit: Error in newterm"));
          if ((set_term (screen)) == NULL)
            return (eng_error_out("shinit: Error in set_term")); 

          /* Make cureses windows so we leverage some of there suff */
          /* Start color */
          if (start_color () == ERR)
            return (eng_error_out("shinit: Error in start_color"));

          /* Allocate the storage for the screens */
          for (i = 0; i < SH_NUM_SCREEN; i++)
            {
              Sh_scr_ptr = Sh_scr_data[i].window[0];

              if ((Sh_scr_ptr->current_win = newwin (35, 98, 0, 0)) == NULL)
                return (eng_error_out("shinit: Error in newwin: Sh_scr_data[i].current_win"));
            }

          if ((Sh_mesg_win = newwin (2, 98, 34, 0)) == NULL)
            return (eng_error_out("shinit: Error in newwin: Sh_mesg_win"));
        }
      else
        {
          eng_debug ("shinit: Init text terminal info");
          initscr ();                                          /* Initialize curses */
          //      mousemask (ALL_MOUSE_EVENTS, NULL);
          noecho ();                                           /* Do not echo keys*/
          raw ();                                             /* Send us one key at a time */
          if (start_color () == ERR)                           /* Start color */
            return (eng_error_out ("shinit - start_color failed"));

          getmaxyx (stdscr, row, colum);
          for (i = 0; i < SH_NUM_SCREEN; i++)                           /* Make available three win's */
            {
              Sh_scr_ptr = Sh_scr_data[i].window[0];

              Sh_scr_ptr->current_win = newwin (row - 2, colum, 0, 0);
              keypad (Sh_scr_ptr->current_win, TRUE);
              memset (&Sh_scr_ptr->temp_header, ' ', sizeof (struct _tmhdr));
            }
          Sh_mesg_win = newwin (2, colum, row - 2, 0);

          if (init_pair (16, COLOR_BLACK, COLOR_YELLOW) == ERR)        /* Reserve 16 for reverse active field highlight */
            return (eng_error_out("shinit: Error in init_pair"));
          if (init_pair (15, COLOR_WHITE, COLOR_RED) == ERR)           /* Reserve 15 for reverse select field highlight */
            return (eng_error_out("shinit: Error in init_pair"));
        }

       /* Point to the first screen */
      Sh_scr_ptr = Sh_scr_data[0].window[0];

      /* Set colors */
      wbkgdset (Sh_scr_ptr->current_win, ' ' | sh603 (Sh_def_sys_protected, Sh_def_sys_background, SH_PRO_TYPE));
      if (sh611 (Sh_scr_ptr->current_win, Sh_def_sys_protected) < 0)
        return (-1);

      /* Clear message line */
      sh107 (0);

      /* Write a blank screen */
      eng_wrefresh (Sh_scr_ptr->current_win);
    }

  Sh_flags.init = ON;                                      /* set flag for initialzation */
  Sh_ins_toggle = ON;                                      /* Make insert default */
  eng_debug ("shinit: Done");
  return (0);                                                  /* return to mainline */
}

/*
  extsh  -  EXIT SCREEN HANDLER

  Prepares for screen handler exit by sending ending sequence to terminal,
  releasing any held core, and resetting terminal characteristics.
*/

void
extsh ()
{
  char   *next;
  int     i;

  nice (20);                                               /* besure to be back to normal */

  eng_debug ("extsh - EXIT SCREEN HANDLER");

  if (Sh_flags.screen_mode == SH_WEB)
   {
     FCGX_Accept_r (&Sh_fcgi_io);
     FCGX_FPrintF (Sh_fcgi_io.out, "%s!#CMD#!05", Sh_html_header.data);
     FCGX_Finish_r (&Sh_fcgi_io);
     FCGX_FClose (Sh_fcgi_io.out);
     FCGX_ShutdownPending();
   }

  msgctl (Sh_mesg_id, IPC_RMID, 0);                        /* kill message queue */

  for (i = 0; i < SH_NUM_SCREEN; i++)
  {
    Sh_scr_ptr = Sh_scr_data[i].window[0];

    /* This will free most of the core for us
    also it will completely free core on any
    popup */
    sh503 (0);

    /* We have to free the base core
    by hand */
    delwin (Sh_scr_ptr->current_win);

    if (Sh_scr_ptr->input_buffer)
      free (Sh_scr_ptr->input_buffer);

    while (Sh_scr_ptr->sav_pdat_beg)
    {
      next = (char *) Sh_scr_ptr->sav_pdat_beg->tmpnxt;
      free (Sh_scr_ptr->sav_pdat_beg);
      Sh_scr_ptr->sav_pdat_beg = (struct _savp *) next;
    }
    free (Sh_scr_ptr);
  }
  delwin (Sh_mesg_win);

  while (Sh_beg_config_keys)
  {
    next = (char *) Sh_beg_config_keys->next_eng_config_keys_list;
    free (Sh_beg_config_keys->config_keys);
    free (Sh_beg_config_keys);
    Sh_beg_config_keys = (struct _eng_config_keys_list *) next;
  }

  fclose (Sh_log_fd);
  endwin ();

   /* If there was a message output to screen */
  if ((*Sh_mesg1 != ' ' || *Sh_mesg2 != ' ') && (Sh_flags.screen_mode == SH_TEXT))
    {
      printf ("%s\n", make_string (Sh_mesg1, SH_TMESG_LEN,1,0,0,0));
      printf ("%s\n", make_string (Sh_mesg2, SH_TMESG_LEN,1,0,0,0));
    }

  Sh_flags.init = OFF;                                      /* set flag for initialzation */
}

/* parse_query */
void parse_query ()
{
  char    *string;    /* String from url */
  char    *disp;      /* strtok dispisition */
  char    *token;     /* Pointer to next token */
  char    *token_sav;     /* Pointer to last token */

  if ((string = FCGX_GetParam ("QUERY_STRING", Sh_fcgi_io.envp)) == NULL)
    {
      eng_debug ("parse_query: No QUERY_STRING FOUND");
      return;
    }

  if ((token = strtok_r(string, "=&", &disp)) == NULL)       /* intial separation */
    return;

  do
    {
      /* Get the length of the token */
      if (strlen(token) <= 0)
        return;

      /* Save it for later */
      token_sav = token;

      if ((token = strtok_r(NULL, "=&", &disp)) == NULL)
        return;
      setenv (token_sav, token, 1);
    }
  while((token = strtok_r(NULL, "=&", &disp)) != NULL);
}

