/*
*  C Implementation: sh102
*
* Description: Screen control
*
*
* Author: Kevin Lloyd <kalloyd@gmail.com>, (C) 2005
*
* Copyright: See COPYING file that comes with this distribution
*
*/

int
tish (char *parm)
{
   /* point to user's request */
  Sh_sh = (struct _sh_parm *) parm;

  eng_debug ("shmain - sh handler: screen: %8.8s function: %2.2s", Sh_sh->screen, Sh_sh->function.ci);

  /* default to good status */
  Sh_sh->status.ui = ENG_SH_OK;

  /* Set screen as detached */
  if (Sh_sh->function.ui == ENG_SH_DS)
    {
      Sh_flags.detached = ON;
      return (0);
    }

  /* See if screen has been inited */
  if (Sh_sh->function.ui == ENG_SH_ST)
  {
    if (Sh_flags.init)
      return (0);
    else
      return (-1);
  }

   /* only allow logging if detached */
  if (Sh_flags.detached == ON && Sh_sh->function.ui != ENG_SH_EN && Sh_sh->function.ui != ENG_SH_IN)
  {
    if (Sh_sh->function.ui == ENG_SH_WM || Sh_sh->function.ui == ENG_SH_WT || Sh_sh->function.ui == ENG_SH_AL)
      Sh_sh->function.ui = ENG_SH_WL;
    else
      return (0);
  }

  /* If we had not init'ed then write raw */
  if ((Sh_sh->function.ui == ENG_SH_WM || Sh_sh->function.ui == ENG_SH_WL || Sh_sh->function.ui == ENG_SH_AL) && !Sh_flags.init)
    return (sh108 ());

   /* if initialization not done */
  if (!Sh_flags.init)
    {
      /* If we never started just return */
      if (Sh_sh->function.ui == ENG_SH_EN)
        return (0);
      
      if (shinit () < 0)
        return (-1);
    }

  /* Init only just return */
  if (Sh_sh->function.ui == ENG_SH_IN)
    return (0);

  /* CP/PH/OTHER have there own screen core */
  if (memcmp (Sh_sh->id, "!##!", 4) != 0)
    Sh_current_scr_core = 0;
  else if (memcmp (Sh_sh->id, "!##!CP", 6) == 0)
    Sh_current_scr_core = 1;
  else if (memcmp (Sh_sh->id, "!##!PH", 6) == 0)
    Sh_current_scr_core = 2;
  else if (memcmp (Sh_sh->id, "!##!EX", 6) == 0)
    Sh_current_scr_core = 3;
  else
    return (-1);

  /* Point to the cores */
  Sh_scr_ptr = Sh_scr_data[Sh_current_scr_core].window[Sh_scr_data[Sh_current_scr_core].cur_window];

  /* set at 'super' priority */
  nice (-19);

  switch (Sh_sh->function.ui)
    {
    case ENG_SH_RD:                                              /* Read Screen */
      sh101 (0);
      break;

    case ENG_SH_RC:                                              /* Read Screen with one keystroke control */
      sh101 (1);
      break;

    case ENG_SH_RR:                                              /* Re-Read */
      sh202 ();
      break;

    case ENG_SH_WT:                                              /* Write Screen */
      sh102 ();
      break;

    case ENG_SH_RW:                                              /* Re-Write screen */
      sh111 ();
      break;

    case ENG_SH_SN:                                              /* Inform webui of fork */
      sh302 ();
      break;

    case ENG_SH_CN:                                              /* Connect screen */
      sh303 ();
      break;

    case ENG_SH_WM:                                             /* Write message line */
      sh107 (1);
      break;

    case ENG_SH_AL:                                             /* Write message box */
      sh110 ();
      break;

    case ENG_SH_WL:                                              /* Write to Log */
      sh109 ();
      break;

    case ENG_SH_CM:                                              /* Clear Message line */
      sh107 (0);
      break;

    case ENG_SH_AT:                                              /* Set Attribute */
      sh103 ();
      break;

    case ENG_SH_AC:                                              /* Reset attributes to original */
      sh104 ();
      break;

    case ENG_SH_AS:                                              /* Save attributes */
      sh105 ();
      break;

    case ENG_SH_AR:                                              /* Restore attributes */
      sh106 ();
      break;

    case ENG_SH_LI:                                              /* Login */
      {
        sh113 ();
        break;
      }
      
    case ENG_SH_GF:                                            /* Get/set font settings */
    case ENG_SH_GS:
    case ENG_SH_GR:
    case ENG_SH_GC:
    case ENG_SH_SF:
    case ENG_SH_SS:
    case ENG_SH_SR:
    case ENG_SH_SC:
      sh112 ();
      break;

    case ENG_SH_EN:                                              /* END SCREEN HANDLER */
      extsh ();
      break;
    }

  /* return to normal priority */
  nice (19);

  /* Set back to main screen */
  Sh_scr_ptr = Sh_scr_data[0].window[Sh_scr_data[0].cur_window];
  
  eng_debug ("shmain - MAIN (exit)");
  return (0);
}
